@@ -29,9 +29,16 @@
 /*******************************************************************/
 
 #include "conf.h"
-#include <stdio.h>
 
-/*#if defined(STDC_HEADERS) || defined(SUN4)*/
+#include <stdio.h>
+#include <math.h>
+
+
+
+#ifndef _GETOPT_H
+#include "../pub/getopt.h"
+#endif
+
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
@@ -39,7 +46,6 @@
 #include <string.h>
 #include <ctype.h>
 #include <limits.h>
-
 #ifdef HAVE_UNAME
 #include <sys/utsname.h>
 #else
@@ -148,6 +154,18 @@ extern int errno;
 #include <unistd.h>
 #endif
 
+
+#ifdef DARWIN
+#include <sys/malloc.h>
+#include <sys/paths.h>
+#endif
+
+#ifdef HAVE_SYS_MALLOC_H
+#ifdef DARWIN
+#include <sys/malloc.h>
+#include <sys/paths.h>
+#endif
+#else
 #ifdef HAVE_MALLOC_H
 #ifndef OPENBSD
 #ifdef __FreeBSD__
@@ -157,6 +175,7 @@ extern int errno;
 #endif
 #endif
 #endif
+#endif
 
 #include <fcntl.h>
 
@@ -190,6 +209,8 @@ extern int errno;
 #include <pwd.h>
 #include <grp.h>
 
+
+
 #ifdef HAVE_SYS_SOCKIO_H
 # include <sys/sockio.h>
 #endif
@@ -218,7 +239,9 @@ extern int errno;
 #endif
 #endif
 
-#ifdef HAVE_RXPOSIX_H
+#ifdef HAVE_PCRE
+# include <pcreposix.h>
+#elif HAVE_RXPOSIX_H
 # include <rxposix.h>
 #elif  HAVE_REGEX_H
 # include <regex.h>
@@ -231,70 +254,100 @@ extern int errno;
 #endif
 
 
+#ifdef HAVE_PTHREAD_H
+# define __USE_GNU 1
+
+# include <pthread.h>
+# ifndef _SC_THREAD_STACK_MIN
+# define _SC_THREAD_STACK_MIN PTHREAD_STACK_MIN
+# endif
+#endif
+
+#ifdef HAVE_SCHED_H
+# include <sched.h>
+#endif
+
+
 /*******************************************************************/
 /* Various defines                                                 */
 /*******************************************************************/
 
 #define true  1
 #define false 0
-#define bufsize 4096
-#define buffer_margin 32
-#define maxvarsize 1024
-#define maxlinksize 256
-#define maxlinklevel 4
-#define maxargs 31
-#define cfmaxiplen 64       /* numerical ip length */
-#define noproccols 16
-#define hashtablesize 1021  /* prime number */
-#define macroalphabet 61    /* a-z, A-Z plus a bit */
-#define maxshellargs 30
-#define samemode 0
-#define sameowner (uid_t)-1
-#define unknown_owner (uid_t)-2
-#define samegroup (gid_t)-1
-#define unknown_group (gid_t)-2
-#define cfnosize    -1
-#define extra_space 8      /* pads items during AppendItem for eol handling in editfiles */
-#define cfinfinity (int)999999999
-#define ticksperday 86400 /* 60 * 60 *24 */
-#define cf_not_connected -1
-#define recursion_limit 100
-#define cf_monday_morning 342000
+#define CF_BUFSIZE 4096
+#define CF_EXPANDSIZE 2*CF_BUFSIZE
+#define CF_ALLCLASSSIZE 4*CF_BUFSIZE
+#define CF_BUFFERMARGIN 32
+#define CF_BLOWFISHSIZE 16
+#define CF_MAXVARSIZE 1024
+#define CF_NONCELEN CF_BUFSIZE/16
+#define CF_MAXLINKSIZE 256
+#define CF_MAXLINKLEVEL 4
+#define CF_MAXARGS 31
+#define CF_MAXFARGS 8
+#define CF_MAX_IP_LEN 64       /* numerical ip length */
+#define CF_PROCCOLS 16
+#define CF_HASHTABLESIZE 4969 /* prime number */
+#define CF_MACROALPHABET 61    /* a-z, A-Z plus a bit */
+#define CF_MAXSHELLARGS 30
+#define CF_SAMEMODE 0
+#define CF_SAME_OWNER (uid_t)-1
+#define CF_UNKNOWN_OWNER (uid_t)-2
+#define CF_SAME_GROUP (gid_t)-1
+#define CF_UNKNOWN_GROUP (gid_t)-2
+#define CF_NOSIZE    -1
+#define CF_EXTRASPC 8      /* pads items during AppendItem for eol handling in editfiles */
+#define CF_INFINITY (int)999999999
+#define CF_TICKS_PER_DAY 86400 /* 60 * 60 *24 */
+#define CF_NOT_CONNECTED -1
+#define CF_RECURSION_LIMIT 100
+#define CF_MONDAY_MORNING 342000
+#define CF_NOVAL -0.7259285297502359
 
-#define exec_ifelapsed 5
-#define exec_expireafter 10
+#define CF_METHODEXEC 0
+#define CF_METHODREPLY  1
+#define CF_EXEC_IFELAPSED 5
+#define CF_EXEC_EXPIREAFTER 10
 
 
 /* Need this to to avoid conflict with solaris 2.6 and db.h */
 
 #ifdef SOLARIS
-# define u_int32_t uint32_t
-# define u_int16_t uint16_t
-# define u_int8_t uint8_t
+# ifndef u_int32_t
+#  define u_int32_t uint32_t
+#  define u_int16_t uint16_t
+#  define u_int8_t uint8_t
+# endif
 #endif
 
+#include <db.h>
+
 
 /*******************************************************************/
 /* Class array limits                                              */
 /* This is the only place you ever need to edit anything           */
 /*******************************************************************/
 
-#define CLSSATTR 35         /* increase this for each new class added */
-                            /* It defines the array size for class data */
-#define ATTRDIM 3           /* Only used in CLASSATTRUBUTES[][] defn */
+#define CF_CLASSATTR 36         /* increase this for each new class added */
+                                /* It defines the array size for class data */
+#define CF_ATTRDIM 3            /* Only used in CLASSATTRUBUTES[][] defn */
 
    /* end class array limits */
 
 /*******************************************************************/
 
-#define AVDB_FILE     "cf_averages.db"
-#define STATELOG_FILE "state_log"
-#define ENV_NEW_FILE  "env_data.new"
-#define ENV_FILE      "env_data"
+#define CF_AVDB_FILE     "cf_learning.db"
+#define CF_STATEDB_FILE  "cf_state.db"
+#define CF_LASTDB_FILE   "cf_lastseen.db"
+#define CF_STATELOG_FILE "state_log"
+#define CF_ENVNEW_FILE  "env_data.new"
+#define CF_ENV_FILE      "env_data"
+#define CF_TCPDUMP_COMM "/usr/sbin/tcpdump -t -n -v"
 
-#define CFINPUTSVAR "CFINPUTS"          /* default name for file path var */
-#define CFALLCLASSESVAR "CFALLCLASSES"  /* default name for CFALLCLASSES env */
-#define INFINITERECURSE -99             /* code used to signify inf in recursion */
+
+#define CF_INPUTSVAR "CFINPUTS"          /* default name for file path var */
+#define CF_ALLCLASSESVAR "CFALLCLASSES"  /* default name for CFALLCLASSES env */
+#define CF_INF_RECURSE -99             /* code used to signify inf in recursion */
 #define CF_TRUNCATE -1
 #define CF_EMPTYFILE -2
 #define CF_USELOGFILE true              /* synonyms for tidy.c */
@@ -310,6 +363,7 @@ extern int errno;
 #define CF_NOUSER -99
 #define CF_RSA_PROTO_OFFSET 24
 #define CF_PROTO_OFFSET 16
+#define CF_INBAND_OFFSET 8
 #define CF_SMALL_OFFSET 2
 #define CF_MD5_LEN 16
 #define CF_SHA_LEN 20
@@ -317,9 +371,13 @@ extern int errno;
 #define CF_DONE 't'
 #define CF_MORE 'm'
 
-#define CFFAILEDSTR "BAD: Host authentication failed. Did you forget the domain name?"
-#define CFCHANGEDSTR1 "BAD: File changed "   /* Split this so it cannot be recognized */
-#define CFCHANGEDSTR2 "while copying"
+#ifndef ERESTARTSYS
+# define ERESTARTSYS EINTR
+#endif
+
+#define CF_FAILEDSTR "BAD: Host authentication failed. Did you forget the domain name or IP/DNS address registration (for ipv4 or ipv6)?"
+#define CF_CHANGEDSTR1 "BAD: File changed "   /* Split this so it cannot be recognized */
+#define CF_CHANGEDSTR2 "while copying"
 
 #define CF_START_DOMAIN "undefined.domain"
 
@@ -329,6 +387,7 @@ extern int errno;
 
 #define Verbose if (VERBOSE || DEBUG || D2) printf
 #define EditVerbose  if (EDITVERBOSE || DEBUG || D2) printf
+#define Debug4  if (D4) printf
 #define Debug3  if (D3 || DEBUG || D2) printf
 #define Debug2  if (DEBUG || D2) printf
 #define Debug1  if (DEBUG || D1) printf
@@ -344,12 +403,12 @@ extern int errno;
 
 /*****************************************************************************/
 
-#define GRAINS   64
+#define CF_GRAINS   64
 #define ATTR     11
+#define CF_NETATTR   7 /* icmp udp dns tcpsyn tcpfin tcpack */
 #define PH_LIMIT 10
-#define CFWEEK   (7.0*24.0*3600.0)
-#define MEASURE_INTERVAL (5.0*60.0)
-
+#define CF_WEEK   (7.0*24.0*3600.0)
+#define CF_MEASURE_INTERVAL (5.0*60.0)
 
 struct Averages
    {
@@ -369,7 +428,29 @@ struct Averages
    double var_loadavg;
    double var_incoming[ATTR];
    double var_outgoing[ATTR];
-   double var_pH[PH_LIMIT];      
+   double var_pH[PH_LIMIT];
+
+      /* tcpdump data - tag them on here ... */
+      
+   double expect_netin[CF_NETATTR];
+   double expect_netout[CF_NETATTR];
+   double var_netin[CF_NETATTR];
+   double var_netout[CF_NETATTR];
+   };
+
+
+struct LastSeen
+   {
+   double expect_lastseen;
+   time_t lastseen;      
+   };
+
+
+
+struct LockData
+   {
+   pid_t pid;
+   time_t time;
    };
 
 /*******************************************************************/
@@ -391,6 +472,8 @@ struct Averages
    ST_NBLOCKS: Number of 512-byte blocks in the file
    (including indirect blocks). */
 
+
+
 #ifndef HAVE_ST_BLOCKS
 # define ST_BLKSIZE(statbuf) DEV_BSIZE
 # if defined(_POSIX_SOURCE) || !defined(BSIZE) /* fileblocks.c uses BSIZE.  */
@@ -460,6 +543,14 @@ enum cf_filetype
 
 /*******************************************************************/
 
+enum roles
+   {
+   cf_connect,
+   cf_accept
+   };
+
+/*******************************************************************/
+
 struct cfstat
    {
    char             *cf_filename;   /* What file are we statting? */
@@ -478,6 +569,7 @@ struct cfstat
    int               cf_failed;     /* stat returned -1 */
    int               cf_nlink;      /* Number of hard links */
    int               cf_ino;        /* inode number on server */
+   dev_t             cf_dev;        /* device number */
    struct cfstat    *next;
    };
 
@@ -497,7 +589,7 @@ typedef struct cfdir CFDIR;
 struct cfdirent
    {
    struct dirent *cf_dirp;
-   char   d_name[bufsize];   /* This is bigger than POSIX */
+   char   d_name[CF_BUFSIZE];   /* This is bigger than POSIX */
    };
 
 
@@ -509,6 +601,14 @@ enum cfsizes
    cfpercent
    };
 
+/*******************************************************************/
+
+enum statepolicy
+   {
+   cfreset,        /* Policy when trying to add already defined persistent states */
+   cfpreserve
+   };
+
 
 /*******************************************************************/
 
@@ -526,9 +626,29 @@ enum builtin
    fn_execresult,
    fn_returnszero,
    fn_iprange,
+   fn_hostrange,
    fn_isdefined,
    fn_strcmp,
-   fn_showstate
+   fn_regcmp,
+   fn_showstate,
+   fn_friendstat,
+   fn_readfile,
+   fn_returnvars,
+   fn_returnclasses,
+   fn_syslog,
+   fn_setstate,
+   fn_unsetstate,
+   fn_module,
+   fn_adj,
+   fn_readarray,
+   fn_readtable,
+   fn_readlist,
+   fn_selectpl,
+   fn_selectpn,
+   fn_selectpna,
+   fn_greaterthan,
+   fn_lessthan,
+   fn_readtcp
    };
 
 /*******************************************************************/
@@ -554,6 +674,7 @@ enum actions
    import,
    shellcommands,
    disable,
+   rename_disable,
    makepath,
    ignore,
    broadcast,
@@ -566,7 +687,9 @@ enum actions
    acls,
    interfaces,
    filters,
-   strategies
+   strategies,
+   packages,
+   methods
    };
 
 /*******************************************************************/
@@ -603,6 +726,7 @@ enum classes
    cfsco,
    darwin,
    ux4800,
+   qnx,
    unused1,
    unused2,
    unused3
@@ -628,8 +752,9 @@ enum fileactions
 
 /*******************************************************************/
 
-enum fileattr  /* See COMMATTRIBUTES[] in globals.c  for matching entry */
+enum commattr  /* See COMMATTRIBUTES[] in globals.c  for matching entry */
    {
+   cffindertype,
    cfrecurse,
    cfmode,
    cfowner,
@@ -643,6 +768,7 @@ enum fileattr  /* See COMMATTRIBUTES[] in globals.c  for matching entry */
    cfforce,
    cfforcedirs,
    cfforceipv4,
+   cfforcereplyto,
    cfbackup,
    cfrotate,
    cfsize,
@@ -668,6 +794,7 @@ enum fileattr  /* See COMMATTRIBUTES[] in globals.c  for matching entry */
    cfuseshell,
    cfsetlog,
    cfsetinform,
+   cfsetipaddress,
    cfsetnetmask,
    cfsetbroadcast,
    cfignore,
@@ -693,6 +820,17 @@ enum fileattr  /* See COMMATTRIBUTES[] in globals.c  for matching entry */
    cfcompat,
    cfmountoptions,
    cfreadonly,
+   cfversion,
+   cfcmp,
+   cfpkgmgr,
+   cfxdev,
+   cfretvars,
+   cfretclasses,
+   cfsendclasses,
+   cfifelap,
+   cfexpaft,
+   cfscan,
+   cfnoabspath,
    cfbad                        /* HvB must be as last */		
    };
 
@@ -712,6 +850,7 @@ enum itemtypes
 
 enum vnames 
    {
+   cfversionvar,
    cffaculty,
    cfsite,
    cfhost,
@@ -757,6 +896,7 @@ enum vnames
    cfcr,
    cfn,
    cfdblquote,
+   cfcolon,
    cfquote,
    cfdollar,
    cfrepchar,
@@ -771,17 +911,35 @@ enum vnames
    cfnonattackers,
    cfattackers,
    cfmulticonn,
+   cfarglist,
+   cfmethodname,
+   cfmethodpeers,
    cftrustkeys,
    cfdynamic,
    cfallowusers,
    cfskipverify,
    cfdefcopy,
    cfredef,
+   cfdefpkgmgr,
    nonexistentvar
    };
 
 /*******************************************************************/
 
+enum methproto
+   {
+   cfmeth_name,
+   cfmeth_file,
+   cfmeth_time,
+   cfmeth_replyto,
+   cfmeth_sendclass,
+   cfmeth_attacharg,
+   cfmeth_isreply,
+   badmeths
+   };
+
+/*******************************************************************/
+
 enum resc
    {
    rmountcom,
@@ -809,6 +967,7 @@ enum aseq
    shellcom,
    chkfiles,
    disabl,
+   renam,
    mountresc,
    edfil,
    mountall,
@@ -819,6 +978,8 @@ enum aseq
    tzone,
    mountinfo,
    procs,
+   pkgs,
+   meths,
    non,
    plugin
    };
@@ -829,8 +990,17 @@ enum editnames
    {
    NoEdit,
    DeleteLinesStarting,
+   DeleteLinesNotStarting,
    DeleteLinesContaining,
+   DeleteLinesNotContaining,
    DeleteLinesMatching,
+   DeleteLinesNotMatching,
+   DeleteLinesStartingFileItems,
+   DeleteLinesContainingFileItems,
+   DeleteLinesMatchingFileItems,  
+   DeleteLinesNotStartingFileItems,
+   DeleteLinesNotContainingFileItems,
+   DeleteLinesNotMatchingFileItems,  
    AppendIfNoSuchLine,
    PrependIfNoSuchLine,
    WarnIfNoSuchLine,
@@ -853,6 +1023,7 @@ enum editnames
    SetSearchRegExp,
    LocateLineMatching,
    InsertLine,
+   AppendIfNoSuchLinesFromFile,
    IncrementPointer,
    ReplaceLineWith,
    DeleteToLineMatching,
@@ -882,6 +1053,7 @@ enum editnames
    CommentNLines,
    UnCommentNLines,
    ReplaceAll,
+   ReplaceFirst,
    With,
    SetLine,
    FixEndOfLine,
@@ -898,6 +1070,7 @@ enum editnames
    BeginGroupIfDefined,
    BeginGroupIfNotDefined,
    AutoCreate,
+   WarnIfFileMissing,
    ForEachLineIn,
    EndLoop,
    ReplaceLinesMatchingField,
@@ -921,7 +1094,9 @@ enum editnames
    EditUmask,
    EditUseShell,
    EditFilter,
-   DefineInGroup
+   DefineInGroup,
+   EditIfElapsed,
+   EditExpireAfter
    };
 
 enum RegExpTypes
@@ -986,8 +1161,65 @@ enum modesort
 
 /*******************************************************************/
 
+enum cmpsense   /* For package version comparison */
+   {
+   cmpsense_eq,
+   cmpsense_gt,
+   cmpsense_lt,
+   cmpsense_ge,
+   cmpsense_le,
+   cmpsense_ne
+   };
+
+/*******************************************************************/
+
+enum pkgmgrs    /* Available package managers to query in packages: */
+   {
+   pkgmgr_rpm,
+   pkgmgr_dpkg,
+   pkgmgr_sun,
+   pkgmgr_none
+   };
+
+/*******************************************************************/
+
+enum pkgactions /* What to do with a package if it is found/not found */
+    {
+    pkgaction_install,
+    pkgaction_remove,
+    pkgaction_none
+    };
+
+/*******************************************************************/
+
 typedef char flag;
 
+enum socks
+   {
+   netbiosns,
+   netbiosdgm,
+   netbiosssn,
+   irc,
+   cfengine,
+   nfsd,
+   smtp,
+   www,
+   ftp,
+   ssh,
+   wwws
+   };
+
+enum iptypes
+   {
+   icmp,
+   udp,
+   dns,
+   tcpsyn,
+   tcpack,
+   tcpfin,
+   tcpmisc
+   };
+
 /*******************************************************************/
 
 struct cfagent_connection
@@ -995,18 +1227,20 @@ struct cfagent_connection
    int sd;
    int trust;
    int family;     /* AF_INET or AF_INET6 */
-   char localip[cfmaxiplen];
-   char remoteip[cfmaxiplen];
+   char localip[CF_MAX_IP_LEN];
+   char remoteip[CF_MAX_IP_LEN];
    unsigned char *session_key;
    short error;
    };
 
+
 /*******************************************************************/
 
 struct cfObject
    {
    char *scope;                       /* Name of object (scope) */
-   char *hashtable[hashtablesize];    /* Variable heap  */
+   char *hashtable[CF_HASHTABLESIZE];    /* Variable heap  */
+   char *type[CF_HASHTABLESIZE];         /* scalar or itlist? */
    char *classlist;                   /* Private classes -- ? */
    struct Item *actionsequence;
    struct cfObject *next;
@@ -1037,6 +1271,7 @@ struct Interface
    char done;
    char *scope;
    char *ifdev;
+   char *ipaddress;
    char *netmask;
    char *broadcast;
    char *classes;
@@ -1045,11 +1280,44 @@ struct Interface
 
 /*******************************************************************/
 
+struct Method
+   {
+   char done;
+   char *scope;
+   char           *chdir;
+   char           *chroot;
+   uid_t          uid;
+   gid_t          gid;
+   char           useshell;
+   struct Item   *send_args;
+   struct Item   *send_classes;
+   struct Item   *servers;
+   struct Item   *return_vars;
+   struct Item   *return_classes;
+   char          *file;
+   char          *name;
+   unsigned char  digest[EVP_MAX_MD_SIZE+1];
+   char          *classes;
+   char          *bundle;
+   char          *forcereplyto;
+   char           invitation;
+   int            ifelapsed;
+   int            expireafter;
+   char           log;
+   char           inform;
+   struct Method *next;
+   };
+
+/*******************************************************************/
+
 struct Item
    {
    char done;
    char *name;
    char *classes;
+   int counter;
+   int ifelapsed;
+   int expireafter;
    struct Item *next;
    char *scope;
    };
@@ -1092,6 +1360,8 @@ struct Process
    struct Item    *exclusions;
    struct Item    *inclusions;
    struct Item    *filters;
+   int ifelapsed;
+   int expireafter;
    struct Process *next;
    };
 
@@ -1118,6 +1388,10 @@ struct Tidy
    char         done;       /* Too intensive in Tidy Pattern */
    char         *scope;
    char        *path;
+   char         xdev;
+   int ifelapsed;
+   int expireafter;
+
    struct Item *exclusions;
    struct Item *ignores;      
    struct TidyPattern *tidylist;
@@ -1139,7 +1413,7 @@ struct Tidy
          char               compress;
          char               travlinks;
          char               dirlinks;          /* k=keep, t=tidy */
-         char               rmdirs;            /* t=true, f=false */
+         char               rmdirs;            /* t=true, f=false, s=sub-only */
          char               searchtype;        /* a, m, c time */
 	 char               log;
 	 char               inform;
@@ -1168,6 +1442,9 @@ struct MiscMount
    char *onto;
    char *options;
    char *classes;
+   int ifelapsed;
+   int expireafter;
+
    struct MiscMount *next;
    };
 
@@ -1182,6 +1459,9 @@ struct UnMount
    char deletedir;  /* y/n - true false */
    char deletefstab;
    char force;
+   int ifelapsed;
+   int expireafter;
+
    struct UnMount *next;
    };
 
@@ -1190,7 +1470,7 @@ struct UnMount
 struct File
    {
    char   done;
-   char *scope;
+   char   *scope;
    char   *path;
    char   *defines;
    char   *elsedef;
@@ -1202,7 +1482,7 @@ struct File
    struct Item *exclusions;
    struct Item *inclusions;
    struct Item *filters;
-   struct Item    *ignores;      
+   struct Item *ignores;      
    char   *classes;
    struct UidList *uid;
    struct GidList *gid;
@@ -1211,6 +1491,9 @@ struct File
    char   log;
    char   compress;
    char   inform;
+   char   xdev;
+   int    ifelapsed;
+   int    expireafter;
    char   checksum;   /* m=md5 n=none */
    u_long plus_flags;    /* for *BSD chflags */
    u_long minus_flags;    /* for *BSD chflags */
@@ -1229,8 +1512,11 @@ struct Disk
    char   force;	/* HvB: Bas van der Vlies */
    int    freespace;
    struct Disk *next;
+   int    ifelapsed;
+   int    expireafter;
    char   log;
    char   inform;
+   char   scanarrivals;
    };
 
 /*******************************************************************/
@@ -1240,6 +1526,7 @@ struct Disable
    char  done;
    char *scope;
    char  *name;
+   char  *destination;
    char  *classes;
    char  *type;
    char  *repository;
@@ -1253,12 +1540,15 @@ struct Disable
    struct Disable *next;
    char   log;
    char   inform;
+   int ifelapsed;
+   int expireafter;
    };
 
 /*******************************************************************/
 
 struct Image
    {
+   char   *cf_findertype; /* Type info for finder */
    char   done;
    char   *scope;
    char   *path;
@@ -1269,7 +1559,7 @@ struct Image
    mode_t minus;
    struct UidList *uid;
    struct GidList *gid;
-   char   *action;                                   /* fix / warn */
+   char   *action;                           /* fix / warn /silent */
    char   *classes;
    char   *defines;
    char   *elsedef;
@@ -1282,11 +1572,16 @@ struct Image
    char   stealth;               /* preserve times on source files */
    char   preservetimes;                 /* preserve times in copy */
    char   backup;
+   char   xdev;
    int    recurse;
    int    makeholes;
    off_t  size;
    char   comp;
    char   purge;
+
+   int ifelapsed;
+   int expireafter;
+
    struct Item *exclusions;
    struct Item *inclusions;
    struct Item *filters;      
@@ -1296,7 +1591,6 @@ struct Image
    struct cfstat *cache;                              /* stat cache */
    struct CompressedArray *inode_cache;              /* inode cache */
  
-   struct in_addr *dns;                      /* Cache gethostbyname */
    int    addrfamily;
    
    struct Image *next;
@@ -1350,7 +1644,11 @@ enum matchtypes
     literalStart,
     literalComplete,
     literalSomewhere,
-    regexComplete
+    regexComplete,
+    NOTliteralStart,
+    NOTliteralComplete,
+    NOTliteralSomewhere,
+    NOTregexComplete
     };
 
 
@@ -1398,6 +1696,8 @@ struct Link
    char   type;
    char   copytype;
    int    recurse;
+   int ifelapsed;
+   int expireafter;
    struct Item *exclusions;
    struct Item *inclusions;
    struct Item *ignores;            
@@ -1413,6 +1713,7 @@ struct Link
 struct Edit
    {
    char done;     /* Have this here, too dangerous in Edlist */
+   char warn;
    char *scope;
    char *fname;
    char *defines;
@@ -1422,6 +1723,8 @@ struct Edit
    char *repository;
    int   recurse;
    char  binary;   /* y/n */
+   int ifelapsed;
+   int expireafter;
    struct Item *ignores;
    struct Item *exclusions;
    struct Item *inclusions;      
@@ -1516,6 +1819,9 @@ struct ShellComm
    char              *defines;
    char              *elsedef;
    char              preview;
+   char              noabspath;
+   int ifelapsed;
+   int expireafter;
    };
 
 /*******************************************************************/
@@ -1533,6 +1839,7 @@ struct Auth
 
 struct Strategy
    {
+   char    done;
    char   *name;
    char   *classes;
    char   type;                 /* default r=random */
@@ -1540,6 +1847,27 @@ struct Strategy
    struct Strategy *next;
    };
 
+/*******************************************************************/
+
+struct Package      /* For packages: */
+   {
+   char              done;
+   char              *scope;
+   char              *name;
+   char              *classes;
+   char              log;
+   char              inform;
+   char              *defines;
+   char              *elsedef;
+   char              *ver;
+   enum cmpsense     cmp;
+   enum pkgmgrs      pkgmgr;
+   enum pkgactions   action;
+   struct Package    *next;
+   int ifelapsed;
+   int expireafter;
+   };
+
 /*******************************************************************/
 /* Ultrix/BSD don't have all these from sys/stat.h                 */
 /*******************************************************************/
