@@ -29,6 +29,8 @@
 /*******************************************************************/
 
 
+#include "../pub/getopt.h"
+
 #if defined HAVE_PTHREAD_H && (defined HAVE_LIBPTHREAD || defined BUILDTIN_GCC_THREAD)
 extern pthread_mutex_t MUTEX_SYSCALL;
 extern pthread_mutex_t MUTEX_LOCK;
@@ -40,6 +42,7 @@ extern char BINDINTERFACE[CF_BUFSIZE];
 extern char *ECGSOCKS[ATTR][2];
 extern char *TCPNAMES[CF_NETATTR];
 extern char **METHODARGV;
+extern int GOTMETHODARGS;
 
 extern struct Item *QUERYVARS;
 extern struct Item *METHODRETURNVARS;;
@@ -269,7 +272,7 @@ extern struct cfObject *VOBJ;
 extern char *PARSEMETHODRETURNCLASSES;
 
 extern struct Item *METHODARGS;
-extern flag  MOUNT_RO;                  /* mount directory readonly */
+extern flag  CF_MOUNT_RO;                  /* mount directory readonly */
 
 extern struct Item *VALERTS;
 extern struct Item *VMOUNTED;
@@ -456,6 +459,9 @@ extern char *VHOSTEQUIV[];
 extern char *VFSTAB[];
 extern char *VMAILDIR[];
 extern char *VNETSTAT[];
+extern char *VROUTE[];
+extern char *VROUTEADDFMT[];
+extern char *VROUTEDELFMT[];
 extern char *VFILECOMM[];
 extern char *ACTIONSEQTEXT[];
 extern char *VEDITNAMES[];
