@@ -452,15 +452,24 @@ while (!feof(pp))
    if (strcmp(name,"ipaddress") == 0)
       {
       Debug("%s/%s\n",name,content);
-      strncpy(VIPADDRESS,content,CF_MAXVARSIZE-1);
+      strncpy(VIPADDRESS,content,17);
       continue;
       }
 
    if (strcmp(name,"EmailMaxLines") == 0)
       {
       Debug("%s/%s\n",name,content);
-      MAXLINES = 20;
-      sscanf(content,"%d",&MAXLINES);
+
+      MAXLINES = 100;
+      if (strcmp(content,"inf") == 0)
+         {
+         MAXLINES = -2;
+         Debug("EmailMaxLines set to infinity\n");
+         }
+      else 
+         {
+         sscanf(content,"%d",&MAXLINES);
+         }
       continue;
       }
 
@@ -987,7 +996,7 @@ if (strlen(MAILFROM) > 0)
 else
    {
    domain[0] = '\0';
-   sscanf(to,"%*[^@]@%.64s",domain);
+   sscanf(to,"%*[^@]@%64s",domain);
    
    if (strlen(domain) > 0)
       {
@@ -1047,7 +1056,7 @@ sent=send(sd,VBUFF,strlen(VBUFF),0);
     }
  else
     {
-    sprintf(VBUFF,"From: cfengine@%s\r\n",VFQNAME);
+    sprintf(VBUFF,"From: %s\r\n",MAILFROM);
     Debug("%s",VBUFF);    
     }
  
