@@ -52,6 +52,7 @@ int  FileFlag = 0;
 int  TRUSTALL = false;
 enum fileoutputlevels  OUTPUTLEVEL = fopl_normal;
 char OUTPUTDIR[CF_BUFSIZE];
+char INCLUDEFILE[CF_BUFSIZE];
 
 struct Item *VCFRUNCLASSES = NULL;
 struct Item *VCFRUNOPTIONHOSTS = NULL;
@@ -70,7 +71,7 @@ int PollServer ARGLIST((char *host, char *options, int storeinfile));
 void SendClassData ARGLIST((int sd, char *sendbuffer));
 void CheckAccess ARGLIST((char *users));
 void cfrunSyntax ARGLIST((void));
-void ReadCfrunConf ARGLIST((void));
+void ReadCfrunConf ARGLIST((char* cfg_fic));
 int ParseHostname ARGLIST((char *hostname, char *new_hostname));
 void FileOutput ARGLIST((FILE *fp, enum fileoutputlevels level, char *message));
 
@@ -231,43 +232,43 @@ for (i = 1; i < argc; i++)
     Debug("Class item: %s\n",ip->name);
     }
 
- ReadCfrunConf(); 
+  /* XXX Initialize workdir for non privileged users */
+
+ strcpy(CFWORKDIR,WORKDIR);
+
+ if (getuid() > 0)
+    {
+    char *homedir;
+    if ((homedir = getenv("HOME")) != NULL)
+       {
+       strncpy(CFWORKDIR,homedir,CF_BUFSIZE-16);
+       strcat(CFWORKDIR,"/.cfagent");
+       }
+    }
+
+ ReadCfrunConf(VCFRUNHOSTS); 
  
  GetNameInfo();
- 
-/* 
-if (uname(&VSYSNAME) == -1)
-   {
-   perror("uname ");
-   printf("cfrun: uname couldn't get kernel name info!!\n");
-   exit(1);
-   }
 
-if ((strlen(VDOMAIN) > 0) && !strchr(VSYSNAME.nodename,'.'))
-   {
-   sprintf(VFQNAME,"%s.%s",VSYSNAME.nodename,VDOMAIN);
-   }
-else
-   {
-   sprintf(VFQNAME,"%s",VSYSNAME.nodename);
-   }   
-*/
+ CfenginePort();
+ StrCfenginePort();
  
-Debug("FQNAME = %s\n",VFQNAME);
+ Debug("FQNAME = %s, WORKDIR = %s\n",VFQNAME,WORKDIR);
+ 
+ sprintf(VPREFIX,"cfrun:%s",VFQNAME);
  
-sprintf(VPREFIX,"cfrun:%s",VFQNAME);
  
 /* Read hosts file */
-
-umask(077);
-strcpy(VLOCKDIR,WORKDIR);
-strcpy(VLOGDIR,WORKDIR); 
-
-OpenSSL_add_all_algorithms();
-ERR_load_crypto_strings();
-CheckWorkDirectories();
-LoadSecretKeys();
-RandomSeed();
+ 
+ umask(077);
+ strcpy(VLOCKDIR,CFWORKDIR);
+ strcpy(VLOGDIR,CFWORKDIR); 
+ 
+ OpenSSL_add_all_algorithms();
+ ERR_load_crypto_strings();
+ CheckWorkDirectories();
+ LoadSecretKeys();
+ RandomSeed();
 }
 
 
@@ -288,6 +289,7 @@ int PollServer(char *host,char *options,int storeinfile)
   struct Image addresses;
 #ifdef HAVE_GETADDRINFO
   struct addrinfo query, *response=NULL, *ap;
+  char strport[16];
 #endif
   
 CONN = NewAgentConn();
@@ -313,12 +315,21 @@ FileVerbose(fp, "Connecting to server %s to port %d with options %s %s\n",parsed
 
  Debug("Using v6 compatible lookup...\n"); 
 
+if (port)
+   {
+   snprintf(strport,15,"%u",port);
+   }
+else
+   {
+   snprintf(strport,15,"%s",STR_CFENGINEPORT);
+   }
+ 
 memset(&query,0,sizeof(struct addrinfo));
 query.ai_family = AF_UNSPEC;
 query.ai_socktype = SOCK_STREAM;
 query.ai_flags = AI_PASSIVE;
  
-if ((err=getaddrinfo(parsed_host,NULL,&query,&response)) != 0)
+if ((err = getaddrinfo(parsed_host,strport,&query,&response)) != 0)
    {
    snprintf(OUTPUT,CF_BUFSIZE,"Unable to lookup %s (%s)",parsed_host,gai_strerror(err));
    CfLog(cferror,OUTPUT,"");
@@ -355,16 +366,8 @@ if (port)
    }
 else
    {
-   if ((server = getservbyname(CFENGINE_SERVICE,"tcp")) == NULL)
-      {
-      CfLog(cferror,"Unable to find cfengine port","getservbyname");
-      FileOutput(fp, fopl_error, "Unable to find cfengine port\n");
-      exit (1);
-      }
-   else
-      {                                                       
-      raddr.sin_port = (unsigned int) server->s_port;
-      } 
+   CfenginePort();
+   raddr.sin_port = SHORT_CFENGINEPORT;
    }
  
 raddr.sin_addr.s_addr = ((struct in_addr *)(hp->h_addr))->s_addr;
@@ -422,7 +425,7 @@ if (!gotkey)
       }
    }
  
-if (!RemoteConnect(parsed_host,forceipv4))
+if (!RemoteConnect(parsed_host,forceipv4,SHORT_CFENGINEPORT,STR_CFENGINEPORT))
    {
    CfLog(cferror,"Couldn't open a socket","socket");
    FileOutput(fp, fopl_error, "Couldn't open a socket\n");
@@ -551,8 +554,7 @@ return true;
 /* Level 2                                                          */
 /********************************************************************/
 
-void ReadCfrunConf()
-
+void ReadCfrunConf(char* cfg_fic)
 { char filename[CF_BUFSIZE], *sp;
   char buffer[CF_MAXVARSIZE], options[CF_BUFSIZE], line[CF_BUFSIZE];
   FILE *fp;
@@ -572,11 +574,11 @@ if (!strchr(VCFRUNHOSTS, '/'))
       }
    else
       {
-      snprintf(filename, CF_BUFSIZE, "%s/inputs/", WORKDIR);
+      snprintf(filename, CF_BUFSIZE, "%s/inputs/", CFWORKDIR);
       }
    }
  
-strcat(filename,VCFRUNHOSTS);
+strcat(filename,cfg_fic);
 
 if ((fp = fopen(filename,"r")) == NULL)      /* Open root file */
    {
@@ -667,6 +669,14 @@ while (!feof(fp))
       continue;
       }
    
+   if (strncmp(line,"include", strlen("include")) == 0)
+      {
+      sscanf(line,"include = %295[^# \n]", INCLUDEFILE);
+      Verbose("cfrun: include = %s\n", INCLUDEFILE);
+      ReadCfrunConf(INCLUDEFILE);
+      continue;
+      }
+   
    if (strncmp(line,"access",6) == 0)
       {
       for (sp = line; (*sp != '=') && (*sp != '\0'); sp++)
@@ -693,6 +703,7 @@ while (!feof(fp))
       {
       continue;
       }
+   
 
 
    if (VCFRUNOPTIONHOSTS != NULL && !IsItemIn(VCFRUNOPTIONHOSTS,buffer))
@@ -886,6 +897,7 @@ void cfrunSyntax()
  printf("      cfrun -v -- -k -- solaris  Local verbose, all solaris, but no copy\n\n");
  printf("cfrun.hosts file syntax:\n");
  printf("# starts a comment\n");
+ printf("include = [file]\t# External cfrun.hosts file to include.\n");
  printf("domain = [domain]\t# Domain to use for connection(s).\n");
  printf("maxchild = [num]\t# Maximum number of children to spawn during run.\n");
  printf("outputdir = [dir]\t# Directory where to put host output files.\n");
@@ -896,37 +908,6 @@ void cfrunSyntax()
  exit(0);
 }
 
-/*********************************************************************/
-
-struct cfagent_connection *NewAgentConn()
-
-{ struct cfagent_connection *ap = (struct cfagent_connection *)malloc(sizeof(struct cfagent_connection));
-
-ap->sd = CF_NOT_CONNECTED;
-ap->family = AF_INET; 
-ap->trust = false;
-ap->localip[0] = '\0';
-ap->remoteip[0] = '\0';
-ap->session_key = NULL;
-return ap;
-};
-
-/*********************************************************************/
-
-void DeleteAgentConn(ap)
-
-struct cfagent_connection *ap;
-
-{
-if (ap->session_key != NULL)
-   {
-   free(ap->session_key);
-   }
-
-free(ap);
-ap = NULL; 
-}
-
 /***************************************************************/
 /* Linking simplification                                      */
 /***************************************************************/
