@@ -54,7 +54,7 @@ if (CONN->sd == CF_NOT_CONNECTED)
    {   
    Debug("Opening server connnection to %s\n",server);
 
-   if (!RemoteConnect(server,ip->forceipv4))
+   if (!RemoteConnect(server,ip->forceipv4,SHORT_CFENGINEPORT,STR_CFENGINEPORT))
       {
       CfLog(cfinform,"Couldn't open a socket","socket");
       if (CONN->sd != CF_NOT_CONNECTED)
@@ -493,7 +493,7 @@ else
 int CopyRegNet(char *source,char *new,struct Image *ip,off_t size)
 
 { int dd, buf_size,n_read = 0,toget,towrite,plainlen,more = true;
-  int last_write_made_hole = 0, done = false,tosend,cipherlen=0;
+ int last_write_made_hole = 0, done = false,tosend,cipherlen=0,value;
   char *buf,in[CF_BUFSIZE],out[CF_BUFSIZE],sendbuffer[CF_BUFSIZE],cfchangedstr[265];
   unsigned char iv[] = {1,2,3,4,5,6,7,8};
   long n_read_total = 0;  
@@ -636,9 +636,24 @@ while (!done)
       return false;      
       }
 
+   value = -1;
+
+   /* Check for mismatch between encryption here and on server - can lead to misunderstanding*/
+   
+   sscanf(buf,"t %d",&value);
+   
+   if ((value > 0) && strncmp(buf+CF_INBAND_OFFSET,"BAD: ",5) == 0)
+      {
+      snprintf(OUTPUT,CF_BUFSIZE*2,"Network access to cleartext %s:%s denied\n",ip->server,source);      
+      CfLog(cfinform,OUTPUT,"");
+      close(dd);
+      free(buf);
+      return false;
+      }
+   
    if (ip->encrypt == 'y')
       {
-      if (!EVP_DecryptUpdate(&ctx,sendbuffer,&plainlen,buf,cipherlen))
+      if (!EVP_DecryptUpdate(&ctx,(unsigned char *)sendbuffer,&plainlen,(unsigned char*)buf,cipherlen))
          {
          Debug("Decryption failed\n");
          return false;
@@ -663,7 +678,7 @@ while (!done)
             CfLog(cfinform,OUTPUT,"");
             close(dd);
             free(buf);
-     return false;      
+            return false;      
             }
          }
       }
@@ -829,41 +844,5 @@ for (i = 0; i < toget; i++)
    }
 }
 
-/*********************************************************************/
-
-struct cfagent_connection *NewAgentConn()
-
-{ struct cfagent_connection *ap;
-
-if ((ap = (struct cfagent_connection *)malloc(sizeof(struct cfagent_connection))) == NULL)
-   {
-   return NULL;
-   }
-
-Debug("New server connection...\n");
-ap->sd = CF_NOT_CONNECTED;
-ap->family = AF_INET; 
-ap->trust = false;
-ap->localip[0] = '\0';
-ap->remoteip[0] = '\0';
-ap->session_key = NULL;
-ap->error = false; 
-return ap;
-};
-
-/*********************************************************************/
-
-void DeleteAgentConn(struct cfagent_connection *ap)
-
-{
-if (ap->session_key != NULL)
-   {
-   free(ap->session_key);
-   }
-
-free(ap);
-ap = NULL; 
-}
-
 
 
