@@ -512,7 +512,7 @@ int IsDefinedClass(char *class)
   /* is currently true, given the defined heap and negations */
 
 {
- Debug4("IsDefinedClass(%s,VADDCLASSES)\n",class);
+Debug4("IsDefinedClass(%s,VADDCLASSES)\n",class);
 
 if (class == NULL)
    {
@@ -691,7 +691,7 @@ while(*sp != '\0')
       {
       atom = cbuff+1;
 
-      Debug4("Checking AND Atom %s?\n");
+      Debug4("Checking AND Atom %s?\n",atom);
       
       cbuff[strlen(cbuff)-1] = '\0';
       
