@@ -84,8 +84,8 @@ return true;
 char *EvaluateFunction(char *f,char *value)
 
 { enum builtin fn;
-  char name[CF_MAXVARSIZE],vargs[CF_BUFSIZE],args[CF_EXPANDSIZE];
-  int negated = false;
+ char *sp,name[CF_MAXVARSIZE],args[CF_EXPANDSIZE];
+  int negated = false,count = 1;
 
 if (*f == '!')
    {
@@ -93,8 +93,29 @@ if (*f == '!')
    f++;
    }
  
-sscanf(f,"%255[^(](%255[^)])",name,vargs);
-ExpandVarstring(vargs,args,NULL); 
+sscanf(f,"%255[^(]",name);
+
+
+strncpy(args,f+strlen(name)+1,CF_EXPANDSIZE);
+
+for (sp = args; *sp != '\0'; sp++)
+   {
+   if (*sp == '(')
+      {
+      count++;
+      }
+
+   if (*sp == ')')
+      {
+      count--;
+      if (count == 0)
+         {
+         *sp = '\0';
+         }
+      }
+   }
+
+
 Debug("HandleFunction: %s(%s)\n",name,args);
 
 switch (fn = FunctionStringToCode(name))
@@ -131,6 +152,12 @@ switch (fn = FunctionStringToCode(name))
    case fn_strcmp:
        HandleStrCmp(args,value);
        break;
+   case fn_greaterthan:
+       HandleGreaterThan(args,value,'+');
+       break;
+   case fn_lessthan:
+       HandleGreaterThan(args,value,'-');
+       break;       
    case fn_regcmp:
        HandleRegCmp(args,value);
        break;
@@ -143,9 +170,14 @@ switch (fn = FunctionStringToCode(name))
    case fn_readfile:
        HandleReadFile(args,value);
        break;
+
    case fn_readarray:
        HandleReadArray(args,value);
        break;
+   case fn_readtcp:
+       HandleReadTCP(args,value);
+       break;
+       
    case fn_readtable:
        HandleReadTable(args,value);
        break;
@@ -159,7 +191,7 @@ switch (fn = FunctionStringToCode(name))
    case fn_selectpna:
        HandleSelectPGroup(args,value);
        break;
-case fn_syslog:
+   case fn_syslog:
        HandleSyslogFn(args,value);
        break;
        
@@ -276,7 +308,7 @@ return (enum builtin) i;
 void GetRandom(char *args,char *value)
 
 { int result,from=-1,to=-1;
-  char argv[CF_MAXFARGS][CF_MAXVARSIZE];
+  char argv[CF_MAXFARGS][CF_EXPANDSIZE];
  
 if (ACTION != control)
    {
@@ -309,12 +341,9 @@ snprintf(value,CF_BUFSIZE,"%u",result);
 void HandleStatInfo(enum builtin fn,char *args,char *value)
 
 { struct stat statbuf;
+  char argv[CF_MAXFARGS][CF_EXPANDSIZE];
 
-if (strchr(args,','))
-   {
-   yyerror("Illegal argument to unary class-function");
-   return;
-   }
+FunctionArgs(args,argv,1);
 
 if (lstat(args,&statbuf) == -1)
    {
@@ -419,6 +448,7 @@ if (!FuzzyHostParse(args))
    strcpy(value,CF_NOCLASS);
    return;
    }
+
 /* VDEFAULTBINSERVER.name is relative domain name */
 /* (see nameinfo.c ~line 145)                     */
 
@@ -441,7 +471,7 @@ return;
 void HandleCompareStat(enum builtin fn,char *args,char *value)
 
 { struct stat frombuf,tobuf;
-  char argv[CF_MAXFARGS][CF_MAXVARSIZE];
+  char argv[CF_MAXFARGS][CF_EXPANDSIZE];
 
 FunctionArgs(args,argv,2); 
 strcpy(value,CF_NOCLASS);
@@ -568,7 +598,7 @@ strcpy(value,CF_NOCLASS);
 
 void HandleSyslogFn(char *args,char *value)
 
-{ char argv[CF_MAXFARGS][CF_MAXVARSIZE];
+{ char argv[CF_MAXFARGS][CF_EXPANDSIZE];
   int priority = LOG_ERR;
 
 FunctionArgs(args,argv,2);
@@ -625,7 +655,7 @@ if (!DONTDO)
 
 void HandleStrCmp(char *args,char *value)
 
-{ char argv[CF_MAXFARGS][CF_MAXVARSIZE];
+{ char argv[CF_MAXFARGS][CF_EXPANDSIZE];
 
 FunctionArgs(args,argv,2); 
  
@@ -641,9 +671,77 @@ else
 
 /*********************************************************************/
 
+void HandleGreaterThan(char *args,char *value,char ch)
+
+{ char argv[CF_MAXFARGS][CF_EXPANDSIZE];
+  double a = CF_NOVAL,b = CF_NOVAL;
+ 
+FunctionArgs(args,argv,2); 
+
+sscanf(argv[0],"%lf",&a);
+sscanf(argv[1],"%lf",&b);
+
+if ((a != CF_NOVAL) && (b != CF_NOVAL)) 
+   {
+   Debug("%s and %s are numerical\n",argv[0],argv[1]);
+   
+   if (ch == '+')
+      {
+      if (a > b)
+         {
+         strcpy(value,CF_ANYCLASS);
+         }
+      else
+         {
+         strcpy(value,CF_NOCLASS);
+         }
+      return;
+      }
+   else
+      {
+      if (a < b)  
+         {
+         strcpy(value,CF_ANYCLASS);
+         }
+      else
+         {
+         strcpy(value,CF_NOCLASS);
+         }
+      return;
+      }
+   }
+
+Debug("%s and %s are NOT numerical\n",argv[0],argv[1]);
+
+if (strcmp(argv[0],argv[1]) > 0)
+   {
+   if (ch == '+')
+      {
+      strcpy(value,CF_ANYCLASS);
+      }
+   else
+      {
+      strcpy(value,CF_NOCLASS);
+      }
+   }
+else
+   {
+   if (ch == '+')
+      {
+      strcpy(value,CF_NOCLASS);
+      }
+   else
+      {
+      strcpy(value,CF_ANYCLASS);
+      }
+   } 
+}
+
+/*********************************************************************/
+
 void HandleRegCmp(char *args,char *value)
 
-{ char argv[CF_MAXFARGS][CF_MAXVARSIZE];
+{ char argv[CF_MAXFARGS][CF_EXPANDSIZE];
   struct Item *list = NULL, *ret; 
 
 FunctionArgs(args,argv,2);
@@ -654,6 +752,8 @@ if (ACTION != groups)
    return;
    }
 
+Debug("List source = (%d)[%s]\n",strlen(argv[1]),argv[1]);
+
 list = SplitStringAsItemList(argv[1],LISTSEPARATOR);
 
 ret = LocateNextItemMatching(list,argv[0]);
@@ -675,7 +775,7 @@ DeleteItemList(list);
 
 void HandleReadFile(char *args,char *value)
 
-{ char argv[CF_MAXFARGS][CF_MAXVARSIZE];
+{ char argv[CF_MAXFARGS][CF_EXPANDSIZE];
   int i,val = 0;
   FILE *fp;
 
@@ -718,7 +818,7 @@ fclose(fp);
 
 void HandleReadArray(char *args,char *value)
 
-{ char argv[CF_MAXFARGS][CF_MAXVARSIZE];
+{ char argv[CF_MAXFARGS][CF_EXPANDSIZE];
   char *sp,*filename=argv[0],*maxbytes=argv[4],*formattype=argv[1];
   char *commentchar=argv[3],*sepchar=argv[2],buffer[CF_BUFSIZE];
   int i=0,val = 0,read = 0;
@@ -791,7 +891,7 @@ while (!feof(fp))
       }
    else if (strcmp(formattype,"textkey") == 0)
       {
-      char argv[CF_MAXFARGS][CF_MAXVARSIZE],lvalue[CF_BUFSIZE];
+      char argv[CF_MAXFARGS][CF_EXPANDSIZE],lvalue[CF_BUFSIZE];
       if (!FunctionArgs(buffer,argv,2))
          {
          break;
@@ -817,7 +917,7 @@ while (!feof(fp))
 
 void HandleReadTable(char *args,char *value)
 
-{  char argv[CF_MAXFARGS][CF_MAXVARSIZE];
+{  char argv[CF_MAXFARGS][CF_EXPANDSIZE];
   char *sp,*filename=argv[0],*maxbytes=argv[4],*formattype=argv[1];
   char *commentchar=argv[3],*sepchar=argv[2],buffer[CF_BUFSIZE];
   int i=0,j=0,val = 0,read = 0;
@@ -893,7 +993,7 @@ while (!feof(fp))
       }
    else if (strcmp(formattype,"textkey") == 0)
       {
-      char argv[CF_MAXFARGS][CF_MAXVARSIZE],lvalue[CF_BUFSIZE];
+      char argv[CF_MAXFARGS][CF_EXPANDSIZE],lvalue[CF_BUFSIZE];
       if (!FunctionArgs(buffer,argv,3))
          {
          break;
@@ -919,7 +1019,7 @@ snprintf(value,CF_BUFSIZE-1,"CF_ASSOCIATIVE_ARRAY%s",args);
 
 void HandleReadList(char *args,char *value)
 
-{ char argv[CF_MAXFARGS][CF_MAXVARSIZE];
+{ char argv[CF_MAXFARGS][CF_EXPANDSIZE];
   char *sp,*filename=argv[0],*maxbytes=argv[3],*formattype=argv[1];
   char *commentchar=argv[2],buffer[CF_BUFSIZE];
   int val = 0;
@@ -992,11 +1092,71 @@ value[strlen(value)-1] = '\0';
 
 /*********************************************************************/
 
+void HandleReadTCP(char *args,char *value)
+
+{ char argv[CF_MAXFARGS][CF_EXPANDSIZE];
+  char *sp,*hostnameip=argv[0],*maxbytes=argv[3],*port=argv[1];
+  char *sendstring=argv[2],buffer[CF_BUFSIZE];
+  int val = 0, n_read = 0;
+  short portnum;
+  struct Item *list = NULL,*ip;
+  FILE *fp;
+
+FunctionArgs(args,argv,4);
+ 
+val = atoi(maxbytes);
+portnum = (short) atoi(port);
+
+if (val > CF_BUFSIZE-1)
+   {
+   snprintf(OUTPUT,CF_BUFSIZE,"Too many bytes to read from TCP port %s@%s",port,hostnameip);
+   CfLog(cferror,OUTPUT,"");
+   }
+
+Debug("Want to read %d bytes from port %d at %s\n",val,portnum,hostnameip);
+
+CONN = NewAgentConn();
+
+if (!RemoteConnect(hostnameip,'n',portnum,port))
+   {
+   CfLog(cferror,"Couldn't open a tcp socket","socket");
+
+   if (CONN->sd != CF_NOT_CONNECTED)
+      {
+      close(CONN->sd);
+      CONN->sd = CF_NOT_CONNECTED;
+      }
+
+   DeleteAgentConn(CONN);
+   return;
+   }
+
+if (strlen(sendstring) > 0)
+   {
+   if (SendSocketStream(CONN->sd,sendstring,strlen(sendstring),0) == -1)
+      {
+      DeleteAgentConn(CONN);
+      return;   
+      }
+   }
+
+if ((n_read = recv(CONN->sd,value,val,0)) == -1)
+   {
+   }
+
+close(CONN->sd);
+DeleteAgentConn(CONN);
+
+/* value set to result */
+}
+
+/*********************************************************************/
+
 void HandleSelectPLeader(char *args,char *value)
 
 { FILE *fp;
   char machine[128],first[128];
-  char argv[CF_MAXFARGS][CF_MAXVARSIZE];
+  char argv[CF_MAXFARGS][CF_EXPANDSIZE];
   char *filename=argv[0],*commentchar=argv[1];
   char *policy=argv[2],*size=argv[3];
   struct Item *list = NULL,*ip;
@@ -1158,7 +1318,7 @@ fclose(fp);
 
 void HandleSelectPGroup(char *args,char *value)
 
-{ char argv[CF_MAXFARGS][CF_MAXVARSIZE],machine[128],first[128];
+{ char argv[CF_MAXFARGS][CF_EXPANDSIZE],machine[128],first[128];
   char *filename=argv[0],*commentchar=argv[1];
   char *policy=argv[2],*size=argv[3];
   struct Item *list = NULL,*ip;
@@ -1302,7 +1462,7 @@ Verbose("This is a method with return value list: (%s)\n",args);
 
 for (ip = SplitStringAsItemList(args,','); ip != NULL; ip=ip->next)
    {
-   AppendItem(&METHODRETURNVARS,args,CLASSBUFF);
+   AppendItem(&METHODRETURNVARS,ip->name,CLASSBUFF);
    }
  
 strcpy(value,"noinstall");
@@ -1351,7 +1511,7 @@ if (PARSING)
    return;
    }
  
-snprintf(buffer,CF_BUFSIZE-1,"%s/state/cf_%s",WORKDIR,args);
+snprintf(buffer,CF_BUFSIZE-1,"%s/state/cf_%s",CFWORKDIR,args);
 
 if (stat(buffer,&statbuf) == 0)
    {
@@ -1362,8 +1522,6 @@ if (stat(buffer,&statbuf) == 0)
       return;
       }
 
-   printf("%s: -----------------------------------------------------------------------------------\n",VPREFIX);
-   printf("%s: In the last 40 minutes, the peak state was:\n",VPREFIX);
    while(!feof(fp))
       {
       char local[CF_BUFSIZE],remote[CF_BUFSIZE];
@@ -1439,7 +1597,10 @@ if (stat(buffer,&statbuf) == 0)
    
    fclose(fp);
    conns--;
-   
+
+   printf("%s: -----------------------------------------------------------------------------------\n",VPREFIX);
+   printf("%s: In the last 40 minutes, the peak state was q = %d:\n",VPREFIX,conns);
+
    if (IsSocketType(args)||IsTCPType(args))
       {
       if (addresses != NULL)
@@ -1552,7 +1713,7 @@ if (dist)
 
 void HandleFriendStatus(char *args,char *value)
 
-{ char argv[CF_MAXFARGS][CF_MAXVARSIZE];
+{ char argv[CF_MAXFARGS][CF_EXPANDSIZE];
   int time = -1;
   
 if ((ACTION != alerts) && PARSING)
@@ -1582,7 +1743,7 @@ strcpy(value,""); /* No reply */
 
 void HandleSetState(char *args,char *value)
 
-{ char argv[CF_MAXFARGS][CF_MAXVARSIZE];
+{ char argv[CF_MAXFARGS][CF_EXPANDSIZE];
   char *name=argv[0],*ttlbuf=argv[1],*policy=argv[2];
   unsigned int ttl = 0;
 
@@ -1623,21 +1784,22 @@ else
 
 void HandleUnsetState(char *args,char *value)
 
-{ char arg1[CF_MAXVARSIZE];
- 
+{ char argv[CF_MAXFARGS][CF_EXPANDSIZE];
+  
 value[0] = '\0';
-OneArg(args,arg1);
+FunctionArgs(args,argv,1);
 
-Debug("HandleUnsetState(%s)\n",arg1); 
-DeletePersistentClass(arg1);
+
+Debug("HandleUnsetState(%s)\n",argv[0]); 
+DeletePersistentClass(argv[0]);
 }
 
 /*********************************************************************/
 
 void HandlePrepModule(char *args,char *value)
 
-{ char argv[CF_MAXFARGS][CF_MAXVARSIZE];
- char ebuff[CF_EXPANDSIZE];
+{ char argv[CF_MAXFARGS][CF_EXPANDSIZE];
+  char ebuff[CF_EXPANDSIZE];
 
 ExpandVarstring(args,ebuff,NULL);
  
@@ -1660,7 +1822,7 @@ else
 
 void HandleAssociation(char *args,char *value)
 
-{ char argv[CF_MAXFARGS][CF_MAXVARSIZE],lvalue[CF_BUFSIZE];
+{ char argv[CF_MAXFARGS][CF_EXPANDSIZE],lvalue[CF_BUFSIZE];
  
 value[0] = '\0';
 FunctionArgs(args,argv,2);
@@ -1676,99 +1838,52 @@ snprintf(value,CF_BUFSIZE-1,"CF_ASSOCIATIVE_ARRAY%s",args);
 /* Level 3                                                           */
 /*********************************************************************/
 
-void OneArg(args,arg1)
+int FunctionArgs(char *args,char arg[CF_MAXFARGS][CF_EXPANDSIZE],int number)
 
-char *args,*arg1;
-
-{ char one[CF_MAXVARSIZE];
-
-memset(one,0,CF_MAXVARSIZE);
- 
-if (strchr(args,','))
-   {
-   yyerror("Illegal argument to unary function");
-   return;
-   }
-
-strcpy(arg1,UnQuote(args));
-}
-
-/*********************************************************************/
-
-int FunctionArgs(char *args,char arg[CF_MAXFARGS][CF_MAXVARSIZE],int number)
-
-{ char argv[CF_MAXFARGS][CF_MAXVARSIZE];
-  char *sp,*start[CF_MAXFARGS];
+{ char *sp,*start[CF_MAXFARGS],exbuf[CF_EXPANDSIZE];
   int count = 0, i;
+  struct Item *ip =  NULL, *ipp;
 
 if (number > CF_MAXFARGS)
    {
    FatalError("Software error: too many function arguments");
    }
-  
+
+ip = ListFromArgs(args);
+
+if (DEBUG)
+   {
+   DebugListItemList(ip);
+   }
+
+count = ListLen(ip);
+
+if (count != number)
+   {
+   if (PARSING)
+      {
+      snprintf(OUTPUT,CF_BUFSIZE,"Function or format of input file requires %d argument items",number);
+      yyerror(OUTPUT);
+      }
+   else
+      {
+      snprintf(OUTPUT,CF_BUFSIZE,"Assignment (%s) with format error - %d/%d expected args",args,count,number);
+      CfLog(cferror,OUTPUT,"");
+      }
+   return false;
+   }
+
+ipp = ip;
+
 for (i = 0; i < number; i++)
    {
-   memset(argv[i],0,CF_MAXVARSIZE);
+   ExpandVarstring(ipp->name,exbuf,NULL);
+   strncpy(arg[i],exbuf,CF_EXPANDSIZE);
+   ipp = ipp->next;
    }
 
-start[0] = args; 
+DeleteItemList(ip);
 
-for (sp = args; *sp != '\0'; sp++)
-   {
-   if (*sp == '\"')
-      {
-      while (*++sp != '\"')
-         {
-         }
-      continue;
-      }
-   
-   if (*sp == '\'')
-      {
-      while (*++sp != '\'')
-         {
-         }
-      continue;
-      }
-   
-   if (*sp == ',')
-      {
-      if (++count > number-1)
-         {
-         break;
-         }
-      
-      start[count] = sp+1;
-      }
-   }
- 
- if (count != number-1)
-    {
-    if (PARSING)
-       {
-       snprintf(OUTPUT,CF_BUFSIZE,"Function or format of input file requires %d argument items",number);
-       yyerror(OUTPUT);
-       }
-    else
-       {
-       snprintf(OUTPUT,CF_BUFSIZE,"Assignment (%s) with format error",args);
-       CfLog(cferror,OUTPUT,"");
-       }
-    return false;
-    }
- 
- for (i = 0; i < number-1; i++)
-    {
-    strncpy(argv[i],start[i],start[i+1]-start[i]-1);
-    }
- 
- sscanf(start[number-1],"%[^)]",argv[number-1]); 
- 
- for (i = 0; i < number; i++)
-    {
-    strncpy(arg[i],UnQuote(argv[i]),100);
-    }
- 
 return true;
 }
 
