@@ -215,6 +215,7 @@ char *TCPNAMES[CF_NETATTR] =
  /* Methods                                                         */
  /*******************************************************************/
 
+  PUBLIC int GOTMETHODARGS = false;
   PUBLIC struct Item *METHODARGS = NULL;
   PUBLIC char ** METHODARGV = NULL;
   PUBLIC int METHODARGC = 0;
@@ -571,6 +572,7 @@ char *TCPNAMES[CF_NETATTR] =
       "selectpartitionneighbors",
       "isgreaterthan",
       "islessthan",
+      "readtcp",
       NULL
       };
 
@@ -830,7 +832,7 @@ char *TCPNAMES[CF_NETATTR] =
   /*
    * HvB: Bas van der Vlies
   */
-  PRIVATE flag MOUNT_RO = false;
+  PRIVATE flag CF_MOUNT_RO = false;
 
   PRIVATE char *COMMATTRIBUTES[] =
      {
