@@ -89,13 +89,10 @@ char VNUMBROADCAST[256];
 
 /*******************************************************************/
 
-void IfConf (vifdev,vnetmask,vbroadcast)
-
-char *vifdev,*vnetmask, *vbroadcast;
+void IfConf (char *vifdev,char *vaddress,char *vnetmask,char *vbroadcast)
 
 { int sk, flags, metric, isnotsane = false;
 
- 
 Verbose("Assumed interface name: %s %s %s\n",vifdev,vnetmask,vbroadcast);
 
 if (!IsPrivileged())                            
@@ -116,7 +113,6 @@ if (vbroadcast[0] == '\0')
    return;
    }
 
-
 strcpy(IFR.ifr_name,vifdev);
 IFR.ifr_addr.sa_family = AF_INET;
 
@@ -143,12 +139,12 @@ if (ioctl(sk,SIOCGIFMETRIC, (caddr_t) &IFR) == -1)   /* Get the routing priority
 
 metric = IFR.ifr_metric;
 
-isnotsane = GetIfStatus(sk,vifdev,vnetmask,vbroadcast);
+isnotsane = GetIfStatus(sk,vifdev,vaddress,vnetmask,vbroadcast);
 
 if (! DONTDO && isnotsane)
    {
-   SetIfStatus(sk,vifdev,vnetmask,vbroadcast);
-   GetIfStatus(sk,vifdev,vnetmask,vbroadcast);
+   SetIfStatus(sk,vifdev,vaddress,vnetmask,vbroadcast);
+   GetIfStatus(sk,vifdev,vaddress,vnetmask,vbroadcast);
    }
 
 close(sk);
@@ -157,10 +153,7 @@ close(sk);
 
 /*******************************************************************/
 
-int GetIfStatus(sk,vifdev,vnetmask,vbroadcast)
-
-int sk;
-char *vifdev,*vnetmask,*vbroadcast;
+int GetIfStatus(int sk,char *vifdev,char *vaddress,char *vnetmask,char *vbroadcast)
 
 { struct sockaddr_in *sin;
   struct sockaddr_in netmask;
@@ -177,7 +170,7 @@ if ((hp = gethostbyname(VSYSNAME.nodename)) == NULL)
    }
 else
    {
-   bcopy(hp->h_addr,&inaddr, hp->h_length);
+   memcpy(&inaddr,hp->h_addr,hp->h_length);
    Verbose("Address given by nameserver: %s\n",inet_ntoa(inaddr));
    }
 
@@ -190,7 +183,17 @@ if (ioctl(sk,SIOCGIFADDR, (caddr_t) &IFR) == -1)   /* Get the device status flag
 
 sin = (struct sockaddr_in *) &IFR.ifr_addr;
 
-
+if (strlen(vaddress) > 0)
+   {
+   if (strcmp(vaddress,(char *)inet_ntoa(sin->sin_addr)) != 0)
+      {
+      CfLog(cferror,"This machine is configured with an address which differs from\n","");
+      CfLog(cferror,"the cfagent configuration\n","");
+      CfLog(cferror,"Don't know what to do yet...\n","");
+      insane = true;
+      }
+   }
+ 
 if (strcmp((char *)inet_ntoa(*(struct in_addr *)(hp->h_addr)),(char *)inet_ntoa(sin->sin_addr)) != 0)
    {
    CfLog(cferror,"This machine is configured with an address which differs from\n","");
@@ -230,7 +233,7 @@ GetBroadcastAddr(inet_ntoa(inaddr),vifdev,vnetmask,vbroadcast);
 
 if (strcmp(VBUFF,VNUMBROADCAST) != 0)
    {
-   snprintf(OUTPUT,bufsize*2,"Broadcast address was %s not %s (should be bit-type %s)\n",VBUFF,VNUMBROADCAST,vbroadcast);
+   snprintf(OUTPUT,CF_BUFSIZE*2,"Broadcast address was %s not %s (should be bit-type %s)\n",VBUFF,VNUMBROADCAST,vbroadcast);
    CfLog(cferror,OUTPUT,"");
    insane = true;
    }
@@ -240,10 +243,7 @@ return(insane);
 
 /*******************************************************************/
 
-void SetIfStatus(sk,vifdev,vnetmask,vbroadcast)
-
-int sk;
-char *vifdev,*vnetmask, *vbroadcast;
+void SetIfStatus(int sk,char *vifdev,char *vaddress,char *vnetmask,char *vbroadcast)
 
 { struct sockaddr_in *sin;
   struct sockaddr_in netmask, broadcast;
@@ -258,6 +258,9 @@ char *vifdev,*vnetmask, *vbroadcast;
       return;
       } 
 
+
+ REWRITE THIS TO USE ifconfig / ipconfig
+
    **********************************/
 
 /* set netmask */
@@ -278,7 +281,7 @@ if (ioctl(sk,SIOCSIFNETMASK, (caddr_t) &IFR) < 0)
    }
 else
    {
-   snprintf(OUTPUT,bufsize*2,"Set Netmask to: %s\n",inet_ntoa(netmask.sin_addr));
+   snprintf(OUTPUT,CF_BUFSIZE*2,"Set Netmask to: %s\n",inet_ntoa(netmask.sin_addr));
    CfLog(cfinform,OUTPUT,"");
    }
 
@@ -303,16 +306,14 @@ if ((char *) sin->sin_addr.s_addr == NULL)
    }
 else
    {
-   snprintf(OUTPUT,bufsize*2,"Set Broadcast address to: %s\n",inet_ntoa(sin->sin_addr));
+   snprintf(OUTPUT,CF_BUFSIZE*2,"Set Broadcast address to: %s\n",inet_ntoa(sin->sin_addr));
    CfLog(cfinform,OUTPUT,"");
    }
 }
 
 /*****************************************************/
 
-void GetBroadcastAddr(ipaddr,vifdev,vnetmask,vbroadcast)
-
-char *ipaddr,*vifdev,*vnetmask,*vbroadcast;
+void GetBroadcastAddr(char *ipaddr,char *vifdev,char *vnetmask,char *vbroadcast)
 
 { unsigned int na,nb,nc,nd;
   unsigned int ia,ib,ic,id;
@@ -362,6 +363,14 @@ void SetDefaultRoute()
 
 { int sk, defaultokay = 1;
   struct sockaddr_in sindst,singw;
+  char oldroute[INET_ADDRSTRLEN];
+  char routefmt[CF_MAXVARSIZE];
+
+/* These OSes have these structs defined but use the route command */
+# if defined DARWIN || defined FREEBSD || defined OPENBSD || defined SOLARIS
+#  undef HAVE_RTENTRY
+#  undef HAVE_ORTENTRY
+# endif
 
 # ifdef HAVE_ORTENTRY
    struct ortentry route;
@@ -373,109 +382,172 @@ void SetDefaultRoute()
 
   FILE *pp;
 
-Verbose("Looking at default route...\n");
-  
+Verbose("Looking for a default route...\n");
+
 if (!IsPrivileged())                            
    {
-   snprintf(OUTPUT,bufsize*2,"Only root can set a default route.");
+   snprintf(OUTPUT,CF_BUFSIZE*2,"Only root can set a default route.");
    CfLog(cfinform,OUTPUT,"");
    return;
    }
 
-if (VDEFAULTROUTE[0] == '\0')
+if (VDEFAULTROUTE == NULL)
    {
-   Debug("cfengine: No default route is defined. Ignoring the routing tables.\n");
+   Verbose("cfengine: No default route is defined. Ignoring the routing tables.\n");
    return;
    }
 
 if ((pp = cfpopen(VNETSTAT[VSYSTEMHARDCLASS],"r")) == NULL)
    {
-   snprintf(OUTPUT,bufsize*2,"Failed to open pipe from %s\n",VNETSTAT[VSYSTEMHARDCLASS]);
+   snprintf(OUTPUT,CF_BUFSIZE*2,"Failed to open pipe from %s\n",VNETSTAT[VSYSTEMHARDCLASS]);
    CfLog(cferror,OUTPUT,"popen");
    return;
    }
 
 while (!feof(pp))
    {
-   ReadLine(VBUFF,bufsize,pp);
+   ReadLine(VBUFF,CF_BUFSIZE,pp);
 
-   if (strncmp(VBUFF,"default",7) == 0)
+   Debug("LINE: %s = %s?\n",VBUFF,VDEFAULTROUTE->name);
+   
+   if ((strncmp(VBUFF,"default",7) == 0)||(strncmp(VBUFF,"0.0.0.0",7) == 0))
       {
-      if (strstr(VBUFF,VDEFAULTROUTE))
-         {
-	 Verbose("cfengine: default route is already set to %s\n",VDEFAULTROUTE);
-         defaultokay = 1;
-         break;
-         }
-      else
-         {
-         CfLog(cferror,"The default packet-route is incorrectly set\n","");
-         CfLog(cferror,"Please correct this manually using route(1).\n","");
-         break;
-         }
+      /* extract the default route */
+      /* format: default|0.0.0.0 <whitespace> route <whitespace> etc */
+      if ((sscanf(VBUFF, "%*[default0. ]%s%*[ ]", &oldroute)) == 1)
+        {
+        if ((strncmp(VDEFAULTROUTE->name, oldroute, INET_ADDRSTRLEN)) == 0)
+          {
+          Verbose("cfengine: default route is already set to %s\n",VDEFAULTROUTE->name);
+          defaultokay = 1;
+          break;
+          }
+        else
+          {
+          Verbose("cfengine: default route is set to %s, but should be %s.\n",oldroute,VDEFAULTROUTE->name);
+          defaultokay = 2;
+          break;
+          }
+        }
+      }
+   else
+      {
+      Debug("No default route is yet registered\n");
+      defaultokay = 0;
       }
-   defaultokay = 0;
    }
 
 cfpclose(pp);
 
-if (defaultokay)
+if (defaultokay == 1)
    {
+   Verbose("Default route is set and agrees with conditional policy\n");
    return;
    }
 
-# if defined HAVE_RTENTRY || defined HAVE_ORTENTRY
- 
-if ((sk = socket(AF_INET,SOCK_RAW,0)) == -1)
+if (defaultokay == 0)
    {
-   if ( VSYSTEMHARDCLASS == linuxx )
-      {
-      Debug ("No raw socket protocol for linux\n");
-      return;
-      }
-   
-   printf("System class %s\n",CLASSTEXT[VSYSTEMHARDCLASS]);
-   perror("cfengine: socket");
-   FatalError("Error in SetDefaultRoute()");
+   AddMultipleClasses("no_default_route");
    }
 
-sindst.sin_family = AF_INET;
-singw.sin_family = AF_INET;
-
-sindst.sin_addr.s_addr = INADDR_ANY;
-singw.sin_addr.s_addr = inet_addr(VDEFAULTROUTE);
-
-route.rt_dst = *(struct sockaddr *)&sindst;      /* This disgusting method is necessary */
-route.rt_gateway = *(struct sockaddr *)&singw;
-route.rt_flags = RTF_GATEWAY;
-
-if (! DONTDO)
+if (IsExcluded(VDEFAULTROUTE->classes))
    {
-   if (ioctl(sk,SIOCADDRT, (caddr_t) &route) == -1)   /* Get the device status flags */
-      {
-      CfLog(cferror,"","ioctl SIOCADDRT:");
-      FatalError("Software error: set default route");
-      }
-
-   CfLog(cferror,"The routing table did not contain a default route.\n","");
-   snprintf(OUTPUT,bufsize*2,"I'm setting it to %s\n",VDEFAULTROUTE);
-   CfLog(cferror,OUTPUT,"");
+   Verbose("cfengine: No default route is applicable. Ignoring the routing tables.\n");
+   return;   
    }
 
-# else
+CfLog(cferror,"The default route is incorrect, trying to correct\n","");
 
-/* Socket routing - don't really know how to do this yet */ 
+if ( strcmp(VROUTE[VSYSTEMHARDCLASS], "-") != 0 )
+   {
 
-Verbose("Sorry don't know how to do routing on this platform\n");
+   Debug ("Using route shell commands to set default route\n");
+   if (defaultokay == 2)
+      {
+      if (! DONTDO)
+         {
+         /* get the route command and the format for the delete argument */
+         snprintf(routefmt,CF_MAXVARSIZE,"%s %s",VROUTE[VSYSTEMHARDCLASS],VROUTEDELFMT[VSYSTEMHARDCLASS]);
+         snprintf(VBUFF,CF_MAXVARSIZE,routefmt,"default",VDEFAULTROUTE->name);
+         if (ShellCommandReturnsZero(VBUFF))
+            {
+            CfLog(cfinform,"Removing old default route","");
+            CfLog(cfinform,VBUFF,"");
+            }
+         else
+            {
+            CfLog(cferror,"Error removing route","");
+            }
+         }
+      }
+   if (! DONTDO)
+      {
+      snprintf(routefmt,CF_MAXVARSIZE,"%s %s",VROUTE[VSYSTEMHARDCLASS],VROUTEADDFMT[VSYSTEMHARDCLASS]);
+      snprintf(VBUFF,CF_MAXVARSIZE,routefmt,"default",VDEFAULTROUTE->name);
+      if (ShellCommandReturnsZero(VBUFF))
+         {
+         CfLog(cfinform,"Setting default route","");
+         CfLog(cfinform,VBUFF,"");
+         }
+      else
+         {
+         CfLog(cferror,"Error setting route","");
+         }
+      }
+   return;
+   }
+else
+   {
+#if defined HAVE_RTENTRY || defined HAVE_ORTENTRY
+   Debug ("Using route ioctl to set default route\n");
+   if ((sk = socket(AF_INET,SOCK_RAW,0)) == -1)
+      {
+      CfLog(cferror,"System class: ", CLASSTEXT[VSYSTEMHARDCLASS]);
+      CfLog(cferror,"","Error in SetDefaultRoute():");
+      perror("cfengine: socket");
+      }
+   else
+      {
+      sindst.sin_family = AF_INET;
+      singw.sin_family = AF_INET;
+
+      sindst.sin_addr.s_addr = INADDR_ANY;
+      singw.sin_addr.s_addr = inet_addr(VDEFAULTROUTE->name);
+
+      route.rt_dst = *(struct sockaddr *)&sindst;      /* This disgusting method is necessary */
+      route.rt_gateway = *(struct sockaddr *)&singw;
+      route.rt_flags = RTF_GATEWAY;
+
+      if (! DONTDO)
+         {
+         if (ioctl(sk,SIOCADDRT, (caddr_t) &route) == -1)   /* Get the device status flags */
+            {
+            CfLog(cferror,"Error setting route:","");
+            perror("cfengine: ioctl SIOCADDRT:");
+            }
+         else
+            {
+            CfLog(cferror,"Setting default route.\n","");
+            snprintf(OUTPUT,CF_BUFSIZE*2,"I'm setting it to %s\n",VDEFAULTROUTE->name);
+            CfLog(cferror,OUTPUT,"");
+            }
+         }
+      }
+#else
+
+   /* Socket routing - don't really know how to do this yet */ 
+
+   Verbose("Sorry don't know how to do routing on this platform\n");
  
-# endif
+#endif
+   }
 }
 
-#else /* NT */
+#else /* NT or IRIX */
 
-void IfConf (vifdev,vnetmask,vbroadcast)
+void IfConf (vifdev,vaddress,vnetmask,vbroadcast)
 
-char *vifdev,*vnetmask, *vbroadcast;
+char *vifdev,*vaddress,*vnetmask, *vbroadcast;
 
 {
 Verbose("Network configuration is not implemented on this OS\n");
