@@ -69,12 +69,6 @@
 
 DB *DBP;
 
-struct LockData
-   {
-   pid_t pid;
-   time_t time;
-   };
-
 /********************************************************************/
 
 void PreLockState()
@@ -85,26 +79,60 @@ void PreLockState()
 
 /********************************************************************/
 
-void HandleSignal(signum)
- 
-int signum;
+void SaveExecLock()
+
+{
+strcpy(SAVELOCK,CFLOCK);
+}
+
+
+/********************************************************************/
+
+void RestoreExecLock()
+
+{
+strcpy(CFLOCK,SAVELOCK);
+}
+
+/********************************************************************/
+
+void HandleSignal(int signum)
  
 {
-snprintf(OUTPUT,bufsize*2,"Received signal %d (%s) while doing [%s]",signum,SIGNALS[signum],CFLOCK);
-Chop(OUTPUT);
-CfLog(cferror,OUTPUT,"");
-snprintf(OUTPUT,bufsize*2,"Logical start time %s ",ctime(&CFSTARTTIME));
-Chop(OUTPUT);
-CfLog(cferror,OUTPUT,"");
-snprintf(OUTPUT,bufsize*2,"This sub-task started really at %s\n",ctime(&CFINITSTARTTIME));
-
-CfLog(cferror,OUTPUT,"");
- 
-if (signum == SIGTERM || signum == SIGINT || signum == SIGHUP || signum == SIGSEGV || signum == SIGKILL)
+if (signum != SIGCHLD)
    {
-   ReleaseCurrentLock();
-   closelog();
-   exit(0);
+   snprintf(OUTPUT,CF_BUFSIZE*2,"Received signal %d (%s) while doing [%s]",signum,SIGNALS[signum],CFLOCK);
+   Chop(OUTPUT);
+   CfLog(cferror,OUTPUT,"");
+   snprintf(OUTPUT,CF_BUFSIZE*2,"Logical start time %s ",ctime(&CFSTARTTIME));
+   Chop(OUTPUT);
+   CfLog(cferror,OUTPUT,"");
+   snprintf(OUTPUT,CF_BUFSIZE*2,"This sub-task started really at %s\n",ctime(&CFINITSTARTTIME));
+   CfLog(cferror,OUTPUT,"");
+   fflush(stdout);
+   
+   if (signum == SIGTERM || signum == SIGINT || signum == SIGHUP || signum == SIGSEGV || signum == SIGKILL|| signum == SIGPIPE)
+      {
+      ReleaseCurrentLock();
+      closelog();
+      exit(0);
+      }
+   else if (signum == SIGUSR1)
+	  {
+	  DEBUG= true;
+	  D2= true;
+	  }
+   else if (signum == SIGUSR2)
+	  {
+	  DEBUG= false;
+	  D2= false;
+	  }
+   else /* zombie cleanup - how hard does it have to be? */
+	  {
+	  }
+
+   /* Reset the signal handler */
+   signal(signum,HandleSignal);
    }
 }
 
@@ -119,11 +147,11 @@ if (IGNORELOCK)
    return;
    }
 
-snprintf(LOCKDB,bufsize,"%s/cfengine_lock_db",VLOCKDIR);
+snprintf(LOCKDB,CF_BUFSIZE,"%s/cfengine_lock_db",VLOCKDIR);
 
 if ((errno = db_create(&DBP,NULL,0)) != 0)
    {
-   snprintf(OUTPUT,bufsize*2,"Couldn't open lock database %s\n",LOCKDB);
+   snprintf(OUTPUT,CF_BUFSIZE*2,"Couldn't open lock database %s\n",LOCKDB);
    CfLog(cferror,OUTPUT,"db_open");
    IGNORELOCK = true;
    return;
@@ -137,7 +165,7 @@ if ((errno = DBP->open(DBP,LOCKDB,NULL,DB_BTREE,DB_CREATE,0644)) != 0)
 if ((errno = DBP->open(DBP,NULL,LOCKDB,NULL,DB_BTREE,DB_CREATE,0644)) != 0)    
 #endif
    {
-   snprintf(OUTPUT,bufsize*2,"Couldn't open lock database %s\n",LOCKDB);
+   snprintf(OUTPUT,CF_BUFSIZE*2,"Couldn't open lock database %s\n",LOCKDB);
    CfLog(cferror,OUTPUT,"db_open");
    IGNORELOCK = true;
    return;
@@ -160,16 +188,13 @@ DBP->close(DBP,0);
 
 /************************************************************************/
 
-int GetLock(operator,operand,ifelapsed,expireafter,host,now)
-
-char *operator, *operand, *host;
-int ifelapsed, expireafter;
-time_t now;
+int GetLock(char *operator,char *operand,int ifelapsed,int expireafter,char *host,time_t now)
 
 { unsigned int pid;
-  int err; 
+  int i, err, sum=0;
   time_t lastcompleted = 0, elapsedtime;
-
+  char c_operator[CF_BUFSIZE],c_operand[CF_BUFSIZE],cc_operator[CF_BUFSIZE],cc_operand[CF_BUFSIZE];
+  
 if (IGNORELOCK)
    {
    return true;
@@ -186,12 +211,29 @@ if (now == 0)
 
 Debug("GetLock(%s,%s,time=%d), ExpireAfter=%d, IfElapsed=%d\n",operator,operand,now,expireafter,ifelapsed);
 
-bzero(CFLOCK,bufsize);
-bzero(CFLAST,bufsize); 
- 
-snprintf(CFLOG,bufsize,"%s/cfengine.%.40s.runlog",VLOGDIR,host);
-snprintf(CFLOCK,bufsize,"lock.%.100s.%.40s.%s.%.100s",VCANONICALFILE,host,operator,operand);
-snprintf(CFLAST,bufsize,"last.%s.100.%.40s.%s.%.100s",VCANONICALFILE,host,operator,operand);
+memset(CFLOCK,0,CF_BUFSIZE);
+memset(CFLAST,0,CF_BUFSIZE);
+
+/* Make local copy in case CanonifyName called - not re-entrant - best fix for now */
+
+strncpy(c_operator,operator,CF_BUFSIZE-1);
+strncpy(c_operand,operand,CF_BUFSIZE-1);
+strncpy(cc_operator,CanonifyName(c_operator),CF_BUFSIZE-1);
+strncpy(cc_operand,CanonifyName(c_operand),CF_BUFSIZE-1);
+
+for (i = 0; operator[i] != '\0'; i++)
+    {
+    sum = (CF_MACROALPHABET * sum + operator[i]) % CF_HASHTABLESIZE;
+    }
+
+for (i = 0; operand[i] != '\0'; i++)
+    {
+    sum = (CF_MACROALPHABET * sum + operand[i]) % CF_HASHTABLESIZE;
+    }
+
+snprintf(CFLOG,CF_BUFSIZE,"%s/cfengine.%.40s.runlog",VLOGDIR,host);
+snprintf(CFLOCK,CF_BUFSIZE,"lock.%.100s.%.40s.%s.%.100s_%d",VCANONICALFILE,host,cc_operator,cc_operand,sum);
+snprintf(CFLAST,CF_BUFSIZE,"last.%.100s.%.40s.%s.%.100s_%d",VCANONICALFILE,host,cc_operator,cc_operand,sum);
  
 if (strlen(CFLOCK) > MAX_FILENAME)
    {
@@ -210,14 +252,14 @@ elapsedtime = (time_t)(now-lastcompleted) / 60;
 
 if (elapsedtime < 0)
    {
-   snprintf(OUTPUT,bufsize*2,"Another cfengine seems to have done %s.%s since I started\n",operator, operand);
+   snprintf(OUTPUT,CF_BUFSIZE*2,"Another cfengine seems to have done [%s.%s] since I started (elapsed=%d)\n",operator,operand,elapsedtime);
    CfLog(cfverbose,OUTPUT,"");
    return false;
    }
 
 if (elapsedtime < ifelapsed)
    {
-   snprintf(OUTPUT,bufsize*2,"Nothing scheduled for %s.%s (%u/%u minutes elapsed)\n",operator,operand,elapsedtime,ifelapsed);
+   snprintf(OUTPUT,CF_BUFSIZE*2,"Nothing scheduled for [%s.%s] (%u/%u minutes elapsed)\n",operator,operand,elapsedtime,ifelapsed);
    CfLog(cfverbose,OUTPUT,"");
    return false;
    }
@@ -231,58 +273,52 @@ if (lastcompleted != 0)
    {
    if (elapsedtime >= expireafter)
       {
-      snprintf(OUTPUT,bufsize*2,"Lock %s expired...(after %u/%u minutes)\n",CFLOCK,elapsedtime,expireafter);
+      snprintf(OUTPUT,CF_BUFSIZE*2,"Lock %s expired (after %u/%u minutes)\n",CFLOCK,elapsedtime,expireafter);
       CfLog(cfinform,OUTPUT,"");
       
       pid = GetLockPid(CFLOCK);
 
       if (pid == -1)
-	 {
-	 snprintf(OUTPUT,bufsize*2,"Illegal pid in corrupt lock %s - ignoring lock\n",CFLOCK);
-	 CfLog(cferror,OUTPUT,"");
-	 }
+         {
+         snprintf(OUTPUT,CF_BUFSIZE*2,"Illegal pid in corrupt lock %s - ignoring lock\n",CFLOCK);
+         CfLog(cferror,OUTPUT,"");
+         }
       else
-	 {
-	 Verbose("Trying to kill expired process, pid %d\n",pid);
-
-	 err = 0;
-	 
-	 if (((err = kill(pid,SIGCONT)) == -1) && (errno != ESRCH))
-	    {
-	    sleep(3);
-	    err=0;
-	    
-	    if ((err = kill(pid,SIGINT)) == -1)
-	       {
-	       sleep(1);
-	       err=0;
-	       
-	       if ((err = kill(pid,SIGTERM)) == -1)
-		  {		 
-		  sleep(5);
-		  err=0;
-		  
-		  if ((err = kill(pid,SIGKILL)) == -1)
-		     {
-		     sleep(1);
-		     }
-		  }
-	       }
-	    }
-
-	 if (err == 0 || errno == ESRCH)
-	    {
-	    LockLog(pid,"Lock expired, process killed",operator,operand);
-	    unlink(CFLOCK);
-	    }
-	 else
-	    {
-	    snprintf(OUTPUT,bufsize*2,"Unable to kill expired cfagent process %d from lock %s, exiting this time..\n",pid,CFLOCK);
-	    CfLog(cferror,OUTPUT,"kill");
-	    
-	    FatalError("");
-	    }
-	 }
+         {
+         Verbose("Trying to kill expired process, pid %d\n",pid);
+         
+         err = 0;
+         
+         if ((err = kill(pid,SIGINT)) == -1)
+            {
+            sleep(1);
+            err=0;
+            
+            if ((err = kill(pid,SIGTERM)) == -1)
+               {   
+               sleep(5);
+               err=0;
+               
+               if ((err = kill(pid,SIGKILL)) == -1)
+                  {
+                  sleep(1);
+                  }
+               }
+            }
+         
+         if (err == 0 || errno == ESRCH)
+            {
+            LockLog(pid,"Lock expired, process killed",operator,operand);
+            unlink(CFLOCK);
+            }
+         else
+            {
+            snprintf(OUTPUT,CF_BUFSIZE*2,"Unable to kill expired cfagent process %d from lock %s, exiting this time..\n",pid,CFLOCK);
+            CfLog(cferror,OUTPUT,"kill");
+            
+            FatalError("");
+            }
+         }
       }
    else
       {
@@ -290,9 +326,9 @@ if (lastcompleted != 0)
       return false;
       }
    }
-
-SetLock();
-return true;
+ 
+ SetLock();
+ return true;
 }
  
 /************************************************************************/
@@ -311,7 +347,8 @@ if (strlen(CFLAST) == 0)
    {
    return;
    }
- 
+
+
 if (DeleteLock(CFLOCK) == -1)
    {
    Debug("Unable to remove lock %s\n",CFLOCK);
@@ -320,11 +357,12 @@ if (DeleteLock(CFLOCK) == -1)
 
 if (PutLock(CFLAST) == -1)
    {
-   snprintf(OUTPUT,bufsize*2,"Unable to create %s\n",CFLAST);
+   snprintf(OUTPUT,CF_BUFSIZE*2,"Unable to create %s\n",CFLAST);
    CfLog(cferror,OUTPUT,"creat");
    return;
    }
 
+ 
 LockLog(getpid(),"Lock removed normally ",CFLOCK,"");
 strcpy(CFLOCK,"no_active_lock");
 }
@@ -349,8 +387,8 @@ if (IGNORELOCK)
    return 1;
    }
 
-bzero(&value,sizeof(value)); 
-bzero(&key,sizeof(key));    
+memset(&value,0,sizeof(value)); 
+memset(&key,0,sizeof(key));    
 
 InitializeLocks();
  
@@ -364,22 +402,22 @@ while ((errno = dbcp->c_get(dbcp, &key, &value, DB_NEXT)) == 0)
    {
    if (value.data != NULL)
       {
-      bcopy(value.data,&entry,sizeof(entry));
-
+      memcpy(&entry,value.data,sizeof(entry));
+      
       elapsedtime = (time_t)(CFSTARTTIME-entry.time) / 60;
       
       if (elapsedtime >= VEXPIREAFTER)      
-	 {
-	 Debug("LOCK-DB-EXPIRED: %s %s\n",ctime(&(entry.time)),key.data);
-	 continue;
-	 }
+         {
+         Debug("LOCK-DB-EXPIRED: %s %s\n",ctime(&(entry.time)),key.data);
+         continue;
+         }
       
       Debug("LOCK-DB-DUMP   : %s %s\n",ctime(&(entry.time)),key.data);
       
       if (strncmp(key.data,"lock",4) == 0)
-	 {
-	 count++;
-	 }
+         {
+         count++;
+         }
       }
    }
 
@@ -405,7 +443,7 @@ if ((mtime = GetLockTime(CFLAST)) == -1)
    
    if (PutLock(CFLAST) == -1)
       {
-      snprintf(OUTPUT,bufsize*2,"Unable to lock %s\n",CFLAST);
+      snprintf(OUTPUT,CF_BUFSIZE*2,"Unable to lock %s\n",CFLAST);
       CfLog(cferror,OUTPUT,"");
       return 0;
       }
@@ -447,28 +485,58 @@ Debug("SetLock(%s)\n",CFLOCK);
 
 if (PutLock(CFLOCK) == -1)
    {
-   snprintf(OUTPUT,bufsize*2,"GetLock: can't open new lock file %s\n",CFLOCK);
+   snprintf(OUTPUT,CF_BUFSIZE*2,"GetLock: can't open new lock file %s\n",CFLOCK);
    CfLog(cferror,OUTPUT,"fopen");
    FatalError("");;
    }
 }
 
+/**********************************************************************/
+
+void SetSignals()
+
+
+{ int i;
+
+ SIGNALS[SIGHUP] = strdup("SIGHUP");
+ SIGNALS[SIGINT] = strdup("SIGINT");
+ SIGNALS[SIGTRAP] = strdup("SIGTRAP");
+ SIGNALS[SIGKILL] = strdup("SIGKILL");
+ SIGNALS[SIGPIPE] = strdup("SIGPIPE");
+ SIGNALS[SIGCONT] = strdup("SIGCONT");
+ SIGNALS[SIGABRT] = strdup("SIGABRT");
+ SIGNALS[SIGSTOP] = strdup("SIGSTOP");
+ SIGNALS[SIGQUIT] = strdup("SIGQUIT");
+ SIGNALS[SIGTERM] = strdup("SIGTERM");
+ SIGNALS[SIGCHLD] = strdup("SIGCHLD");
+ SIGNALS[SIGUSR1] = strdup("SIGUSR1");
+ SIGNALS[SIGUSR2] = strdup("SIGUSR2");
+ SIGNALS[SIGBUS] = strdup("SIGBUS");
+ SIGNALS[SIGSEGV] = strdup("SIGSEGV");
+
+ for (i = 0; i < highest_signal; i++)
+    {
+    if (SIGNALS[i] == NULL)
+       {
+       SIGNALS[i] = strdup("NOSIG");
+       }
+    }
+}
+
 
 /************************************************************************/
 /* Level 3                                                              */
 /************************************************************************/
 
-int PutLock(name)
-
-char *name;
+int PutLock(char *name)
 
 { DBT key,value;
   struct LockData entry;
 
 Debug("PutLock(%s)\n",name);
   
-bzero(&value,sizeof(value)); 
-bzero(&key,sizeof(key));       
+memset(&value,0,sizeof(value)); 
+memset(&key,0,sizeof(key));       
       
 key.data = name;
 key.size = strlen(name)+1;
@@ -482,7 +550,7 @@ if (IGNORELOCK)
  
 if ((errno = DBP->del(DBP,NULL,&key,0)) != 0)
    {
-   Debug("Unable to delete lock [%s]: %s\n",name,db_strerror(errno));
+   Debug("Found no lock [%s]: %s\n",name,db_strerror(errno));
    }
 
 key.data = name;
@@ -491,6 +559,15 @@ entry.pid = getpid();
 entry.time = time((time_t *)NULL); 
 value.data = &entry;
 value.size = sizeof(entry);
+
+#if defined HAVE_LIBPTHREAD || defined BUILDTIN_GCC_THREAD
+ 
+if (pthread_mutex_lock(&MUTEX_LOCK) != 0)
+   {
+   CfLog(cferror,"pthread_mutex_lock failed","pthread_mutex_lock");
+   }
+ 
+#endif
  
 if ((errno = DBP->put(DBP,NULL,&key,&value,0)) != 0)
    {
@@ -499,15 +576,22 @@ if ((errno = DBP->put(DBP,NULL,&key,&value,0)) != 0)
    return -1;
    }      
 
+#if defined HAVE_LIBPTHREAD || defined BUILDTIN_GCC_THREAD
+ 
+if (pthread_mutex_unlock(&MUTEX_LOCK) != 0)
+   {
+   CfLog(cferror,"pthread_mutex_unlock failed","unlock");
+   }
+ 
+#endif
+ 
 CloseLocks(); 
 return 0; 
 }
 
 /************************************************************************/
 
-int DeleteLock(name)
-
-char *name;
+int DeleteLock(char *name)
 
 { DBT key;
 
@@ -516,17 +600,34 @@ if (strcmp(name,"pre-lock-state") == 0)
    return 0;
    }
  
-bzero(&key,sizeof(key));       
+memset(&key,0,sizeof(key));       
       
 key.data = name;
 key.size = strlen(name)+1;
 
+#if defined HAVE_LIBPTHREAD || defined BUILDTIN_GCC_THREAD
+
+if (pthread_mutex_lock(&MUTEX_LOCK) != 0)
+   {
+   CfLog(cferror,"pthread_mutex_lock failed","pthread_mutex_lock");
+   }
+#endif 
+  
 InitializeLocks();
  
 if ((errno = DBP->del(DBP,NULL,&key,0)) != 0)
    {
    Debug("Unable to delete lock [%s]: %s\n",name,db_strerror(errno));
    }
+
+#if defined HAVE_LIBPTHREAD || defined BUILDTIN_GCC_THREAD
+
+if (pthread_mutex_unlock(&MUTEX_LOCK) != 0)
+   {
+   CfLog(cferror,"pthread_mutex_unlock failed","unlock");
+   }
+ 
+#endif
  
 CloseLocks();
 return 0; 
@@ -534,16 +635,14 @@ return 0;
 
 /************************************************************************/
 
-time_t GetLockTime(name)
-
-char *name;
+time_t GetLockTime(char *name)
 
 { DBT key,value;
   struct LockData entry;
 
-bzero(&key,sizeof(key));       
-bzero(&value,sizeof(value));
-bzero(&entry,sizeof(entry));
+memset(&key,0,sizeof(key));       
+memset(&value,0,sizeof(value));
+memset(&entry,0,sizeof(entry));
       
 key.data = name;
 key.size = strlen(name)+1;
@@ -558,7 +657,7 @@ if ((errno = DBP->get(DBP,NULL,&key,&value,0)) != 0)
 
 if (value.data != NULL)
    {
-   bcopy(value.data,&entry,sizeof(entry));
+   memcpy(&entry,value.data,sizeof(entry));
    CloseLocks();
    return entry.time;
    }
@@ -569,15 +668,13 @@ return -1;
 
 /************************************************************************/
 
-pid_t GetLockPid(name)
-
-char *name;
+pid_t GetLockPid(char *name)
 
 { DBT key,value;
   struct LockData entry;
 
-bzero(&value,sizeof(value));
-bzero(&key,sizeof(key));       
+memset(&value,0,sizeof(value));
+memset(&key,0,sizeof(key));       
       
 key.data = name;
 key.size = strlen(name)+1;
@@ -592,7 +689,7 @@ if ((errno = DBP->get(DBP,NULL,&key,&value,0)) != 0)
 
 if (value.data != NULL)
    {
-   bcopy(value.data,&entry,sizeof(entry));
+   memcpy(&entry,value.data,sizeof(entry));
    CloseLocks();
    return entry.pid;
    }
@@ -603,13 +700,10 @@ return -1;
 
 /************************************************************************/
 
-void LockLog(pid,str,operator,operand)
-
-int pid;
-char *str, *operator, *operand;
+void LockLog(int pid,char *str,char *operator,char *operand)
 
 { FILE *fp;
-  char buffer[maxvarsize];
+  char buffer[CF_MAXVARSIZE];
   struct stat statbuf;
   time_t tim;
 
@@ -617,7 +711,7 @@ Debug("LockLog(%s)\n",str);
 
 if ((fp = fopen(CFLOG,"a")) == NULL)
    {
-   snprintf(OUTPUT,bufsize*2,"Can't open lock-log file %s\n",CFLOG);
+   snprintf(OUTPUT,CF_BUFSIZE*2,"Can't open lock-log file %s\n",CFLOG);
    CfLog(cferror,OUTPUT,"fopen");
    FatalError("");
    }
