@@ -215,7 +215,7 @@ if (strlen(name) > CF_MAXVARSIZE)
    return;
    }
 
-snprintf(exp,CF_BUFSIZE,"$(%s)",name);
+snprintf(exp,CF_BUFSIZE,"${%s}",name);
 
 if (strstr(value,exp))
    {
@@ -223,7 +223,7 @@ if (strstr(value,exp))
    return;
    }
 
-snprintf(exp,CF_BUFSIZE,"${%s}",name);
+snprintf(exp,CF_BUFSIZE,"$(%s)",name);
 
 if (strstr(value,exp))
    {
@@ -276,8 +276,11 @@ if (ptr->hashtable[slot] != 0)
       
       if (CompareMacro(name,ptr->hashtable[slot]) == 0)
          {
-         snprintf(VBUFF,CF_BUFSIZE,"Redefinition of macro %s=%s",name,exp);
-         Warning(VBUFF);
+          if (PARSING && !IsItemIn(VREDEFINES,name))
+             {
+             snprintf(VBUFF,CF_BUFSIZE,"Redefinition of macro %s=%s",name,exp);
+             Warning(VBUFF);
+             }
          free(ptr->hashtable[slot]);
          ptr->hashtable[slot] = sp;
          return;
