@@ -14,7 +14,7 @@
  
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   MERCHANTABILITY or FITNESS FOR A PARTICULARmandar PURPOSE.  See the
    GNU General Public License for more details.
  
   You should have received a copy of the GNU General Public License
@@ -367,7 +367,7 @@ if (CHECKSUMDB == NULL)
     
     if (ISCFENGINE)
        {
-       snprintf(OUTPUT,CF_BUFSIZE,"File %s was not in md5 database - new file found",filename);
+       snprintf(OUTPUT,CF_BUFSIZE,"File %s was not in database - new file found",filename);
        CfLog(cfsilent,OUTPUT,"");
        }
     
@@ -532,22 +532,6 @@ Verbose("---------------------------------------------------------------------\n
 
 /*********************************************************************/
 
-void DebugBinOut(char *buffer,int len)
-
-{ char *sp;
-
-Debug("BinaryBuffer[");
- 
-for (sp = buffer; (*sp != '\0') && (sp < buffer+len); sp++)
-   {
-   Debug("%x",*sp);
-   }
- 
-Debug("]\n"); 
-}
-
-/*********************************************************************/
-
 char *ChecksumPrint(char type,unsigned char digest[EVP_MAX_MD_SIZE+1])
 
 { unsigned int i;
@@ -665,38 +649,6 @@ return false;
 }
 
 
-/**********************************************************************/
-
-void SetSignals()
-
-
-{ int i;
-
- SIGNALS[SIGHUP] = strdup("SIGHUP");
- SIGNALS[SIGINT] = strdup("SIGINT");
- SIGNALS[SIGTRAP] = strdup("SIGTRAP");
- SIGNALS[SIGKILL] = strdup("SIGKILL");
- SIGNALS[SIGPIPE] = strdup("SIGPIPE");
- SIGNALS[SIGCONT] = strdup("SIGCONT");
- SIGNALS[SIGABRT] = strdup("SIGABRT");
- SIGNALS[SIGSTOP] = strdup("SIGSTOP");
- SIGNALS[SIGQUIT] = strdup("SIGQUIT");
- SIGNALS[SIGTERM] = strdup("SIGTERM");
- SIGNALS[SIGCHLD] = strdup("SIGCHLD");
- SIGNALS[SIGUSR1] = strdup("SIGUSR1");
- SIGNALS[SIGUSR2] = strdup("SIGUSR2");
- SIGNALS[SIGBUS] = strdup("SIGBUS");
- SIGNALS[SIGSEGV] = strdup("SIGSEGV");
-
- for (i = 0; i < highest_signal; i++)
-    {
-    if (SIGNALS[i] == NULL)
-       {
-       SIGNALS[i] = strdup("NOSIG");
-       }
-    }
-}
-
 /*********************************************************************/
 
 void SetClassesOnScript(char *execstr,char *classes,char *elseclasses,int useshell)
@@ -969,8 +921,12 @@ int linux_redhat_version(void)
 #define REDHAT_ES_ID "Red Hat Enterprise Linux ES"
 #define REDHAT_WS_ID "Red Hat Enterprise Linux WS"
 #define MANDRAKE_ID "Linux Mandrake"
+#define MANDRAKE_10_1_ID "Mandrakelinux"
 #define FEDORA_ID "Fedora Core"
-
+#define WHITEBOX_ID "White Box Enterprise Linux"
+#define CENTOS_ID "CentOS"
+#define SCIENTIFIC_SL_ID "Scientific Linux SL"
+#define SCIENTIFIC_CERN_ID "Scientific Linux CERN"
 #define RELEASE_FLAG "release "
 
 /* We are looking for one of the following strings...
@@ -982,6 +938,9 @@ int linux_redhat_version(void)
  * Linux Mandrake release 7.1 (helium)
  * Red Hat Enterprise Linux ES release 2.1 (Panama)
  * Fedora Core release 1 (Yarrow)
+ * White Box Enterprise linux release 3.0 (Liberation)
+ * Scientific Linux SL Release 4.0 (Beryllium)
+ * CentOS release 4.0 (Final)
  */
 
 #define RH_REL_FILENAME "/etc/redhat-release"
@@ -998,7 +957,7 @@ char *vendor="";
 char *edition="";
 /* Where the numerical release will be found */
 char *release=NULL;
-
+int i;
 int major = -1;
 char strmajor[CF_MAXVARSIZE];
 int minor = -1;
@@ -1044,10 +1003,32 @@ Verbose("Looking for redhat linux info in \"%s\"\n",relstring);
     {
     vendor = "mandrake";
     }
+ else if(!strncmp(relstring, MANDRAKE_10_1_ID, strlen(MANDRAKE_10_1_ID)))
+    {
+    vendor = "mandrake";
+    }
  else if(!strncmp(relstring, FEDORA_ID, strlen(FEDORA_ID)))
     {
     vendor = "fedora";
     }
+ else if(!strncmp(relstring, WHITEBOX_ID, strlen(WHITEBOX_ID)))
+    {
+    vendor = "whitebox";
+    }
+ else if(!strncmp(relstring, SCIENTIFIC_SL_ID, strlen(SCIENTIFIC_SL_ID)))
+    {
+    vendor = "scientific";
+    edition = "sl";
+    }
+ else if(!strncmp(relstring, SCIENTIFIC_CERN_ID, strlen(SCIENTIFIC_CERN_ID)))
+    {
+    vendor = "scientific";
+    edition = "cern";
+    }
+ else if(!strncmp(relstring, CENTOS_ID, strlen(CENTOS_ID)))
+    {
+    vendor = "centos";
+    }
  else
     {
     Verbose("Could not identify OS distro from %s\n", RH_REL_FILENAME);
@@ -1059,6 +1040,16 @@ Verbose("Looking for redhat linux info in \"%s\"\n",relstring);
   * from the infomation above.  We assume that all the strings will
   * have the word 'release' before the numerical release.
   */
+
+  /* Convert relstring to lowercase so that vendors like
+     Scientific Linux don't fall through the cracks.
+   */
+
+ for (i = 0; i < strlen(relstring); i++)
+    {
+    relstring[i] = tolower(relstring[i]);
+    }
+ 
  release = strstr(relstring, RELEASE_FLAG);
  if(release == NULL)
     {
@@ -1239,6 +1230,7 @@ int linux_mandrake_version(void)
 /* We are looking for one of the following strings... */
 #define MANDRAKE_ID "Linux Mandrake"
 #define MANDRAKE_REV_ID "Mandrake Linux"
+#define MANDRAKE_10_1_ID "Mandrakelinux"
 
 #define RELEASE_FLAG "release "
 #define MANDRAKE_REL_FILENAME "/etc/mandrake-release"
@@ -1279,6 +1271,11 @@ char strminor[CF_MAXVARSIZE];
     {
     vendor = "mandrake";
     }
+
+ else if(!strncmp(relstring, MANDRAKE_10_1_ID, strlen(MANDRAKE_10_1_ID)))
+    {
+    vendor = "mandrake";
+    }
  else
     {
     Verbose("Could not identify OS distro from %s\n", MANDRAKE_REL_FILENAME);
