@@ -318,7 +318,7 @@ else
 void GetInterfaceInfo(void)
 
 { int fd,len,i,j;
-  struct ifreq ifbuf[512],ifr, *ifp;
+  struct ifreq ifbuf[8192],ifr, *ifp;
   struct ifconf list;
   struct sockaddr_in *sin;
   struct hostent *hp;
@@ -400,11 +400,14 @@ for (j = 0,len = 0,ifp = list.ifc_req; len < list.ifc_len; len+=SIZEOF_IFREQ(*if
                AddClassToHeap(CanonifyName(inet_ntoa(sin->sin_addr)));
                Debug("Adding hostname %s..\n",hp->h_name);
                AddClassToHeap(CanonifyName(hp->h_name));
-               
-               for (i=0;hp->h_aliases[i]!=NULL;i++)
+
+               if (hp->h_aliases != NULL)
                   {
-                  Debug("Adding alias %s..\n",hp->h_aliases[i]);
-                  AddClassToHeap(CanonifyName(hp->h_aliases[i]));
+                  for (i=0; hp->h_aliases[i] != NULL; i++)
+                     {
+                     Debug("Adding alias %s..\n",hp->h_aliases[i]);
+                     AddClassToHeap(CanonifyName(hp->h_aliases[i]));
+                     }
                   }
                }               
 
@@ -477,6 +480,26 @@ void GetV6InterfaceInfo(void)
            }
         
         break;
+
+    case hp:
+        
+        if ((pp = cfpopen("/usr/sbin/ifconfig -a","r")) == NULL)
+           {
+           Verbose("Could not find interface info\n");
+           return;
+           }
+
+        break;
+
+    case aix:
+        
+        if ((pp = cfpopen("/etc/ifconfig -a","r")) == NULL)
+           {
+           Verbose("Could not find interface info\n");
+           return;
+           }
+
+        break;
         
     default:
         
@@ -590,3 +613,45 @@ if (strstr(VFQNAME,".") == 0 && (strcmp(VDOMAIN,CF_START_DOMAIN) != 0))
 AddClassToHeap(CanonifyName(VDOMAIN));
 DeleteClassFromHeap("undefined_domain");
 }
+
+
+/*******************************************************************/
+
+void SetReferenceTime(int setclasses)
+
+{ time_t tloc;
+ char vbuff[CF_BUFSIZE];
+ 
+if ((tloc = time((time_t *)NULL)) == -1)
+   {
+   CfLog(cferror,"Couldn't read system clock\n","");
+   }
+
+CFSTARTTIME = tloc;
+
+snprintf(vbuff,CF_BUFSIZE,"%s",ctime(&tloc));
+
+Verbose("Reference time set to %s\n",ctime(&tloc));
+
+if (setclasses)
+   {
+   AddTimeClass(vbuff);
+   }
+}
+
+
+/*******************************************************************/
+
+void SetStartTime(int setclasses)
+
+{ time_t tloc;
+ 
+if ((tloc = time((time_t *)NULL)) == -1)
+   {
+   CfLog(cferror,"Couldn't read system clock\n","");
+   }
+
+CFINITSTARTTIME = tloc;
+
+Debug("Job start time set to %s\n",ctime(&tloc));
+}
