@@ -140,6 +140,7 @@ void HandleShowState ARGLIST((char* args,char *value));
 void HandleReadFile ARGLIST((char *args,char *value));
 void HandleReadArray ARGLIST((char *args,char *value));
 void HandleReadList ARGLIST((char *args,char *value));
+void HandleReadTCP ARGLIST((char *args,char *value));
 void HandleReadTable ARGLIST((char *args,char *value));
 void HandleReturnValues ARGLIST((char *args,char *value));
 void HandleReturnClasses ARGLIST((char *args,char *value));
@@ -150,7 +151,7 @@ void HandleSetState ARGLIST((char *args,char *value));
 void HandleUnsetState ARGLIST((char *args,char *value));
 void HandlePrepModule ARGLIST((char *args,char *value));
 void HandleAssociation ARGLIST((char *args,char *value));
-int FunctionArgs ARGLIST((char *args,char argv[CF_MAXFARGS][CF_MAXVARSIZE],int number));
+int FunctionArgs ARGLIST((char *args,char argv[CF_MAXFARGS][CF_EXPANDSIZE],int number));
 void FiveArgs ARGLIST((char *args,char *arg1,char *arg2,char *arg3, char *arg4,char *arg5));
 int IsSocketType ARGLIST((char *s));
 int IsTCPType ARGLIST((char *s));
@@ -182,7 +183,7 @@ void FlushToEnd ARGLIST((int sd, int toget));
 int cfprintf(char *out, int len2, char *in1, char *in2, char *in3);
 struct cfagent_connection *NewAgentConn ARGLIST((void));
 void DeleteAgentConn ARGLIST((struct cfagent_connection *ap));
-int RemoteConnect ARGLIST((char *host,char forceipv4));
+int RemoteConnect ARGLIST((char *host,char forceipv4,short oldport,char *newport));
 
 /* comparray.c */
 
@@ -696,6 +697,8 @@ int OptionIs ARGLIST((char *scope, char *name, short on));
 
 /* methods.c */
 
+void CheckForMethod ARGLIST((void));
+void CheckMethodReply ARGLIST((void));
 void DispatchNewMethod ARGLIST((struct Method *ptr));
 struct Item *GetPendingMethods ARGLIST((int state));
 int ChildLoadMethodPackage ARGLIST((char *name, char *md5));
@@ -745,6 +748,9 @@ void IDClasses ARGLIST((void));
 void AddListSeparator(char *s);
 void ChopListSeparator(char *s);
 
+void SetReferenceTime(int setclasses);
+void SetStartTime(int setclasses);
+
 /* modes.c */
 
 void ParseModeString ARGLIST((char *modestring, mode_t *plusmask, mode_t *minusmask));
