@@ -46,32 +46,36 @@ char *VSKIPFILES[] =
 
 /*********************************************************************/
 
-int SensibleFile(nodename,path,ip)
-
-char *nodename, *path;
-struct Image *ip;
+int SensibleFile(char *nodename,char *path,struct Image *ip)
 
 { int i, suspicious = true;
   struct stat statbuf; 
-  unsigned char *sp, newname[bufsize],vbuff[bufsize];
+  unsigned char *sp, newname[CF_BUFSIZE],vbuff[CF_BUFSIZE];
 
 if (strlen(nodename) < 1)
    {
-   snprintf(OUTPUT,bufsize,"Empty (null) filename detected in %s\n",path);
+   snprintf(OUTPUT,CF_BUFSIZE,"Empty (null) filename detected in %s\n",path);
    CfLog(cferror,OUTPUT,"");
    return false;
    }
 
 if (IsItemIn(SUSPICIOUSLIST,nodename))
    {
-   snprintf(OUTPUT,bufsize,"Suspicious file %s found in %s\n",nodename,path);
-   CfLog(cferror,OUTPUT,"");
-   return false;
+   struct stat statbuf;
+   if (stat(nodename,&statbuf) != -1)
+      {
+      if (S_ISREG(statbuf.st_mode))
+         {
+         snprintf(OUTPUT,CF_BUFSIZE,"Suspicious file %s found in %s\n",nodename,path);
+         CfLog(cferror,OUTPUT,"");
+         return false;
+         }
+      }
    }
-
-if ((strcmp(nodename,"...") == 0) && (strcmp(path,"/") == 0))
+ 
+if (strcmp(nodename,"...") == 0)
    {
-   Verbose("DFS cell node detected in /...\n");
+   Verbose("Possible DFS/FS cell node detected in %s...\n",path);
    return true;
    }
   
@@ -109,21 +113,21 @@ strcat(vbuff,nodename);
 
 if (suspicious && NONALPHAFILES)
    {
-   snprintf(OUTPUT,bufsize,"Suspicious filename %s in %s has no alphanumeric content (security)",CanonifyName(nodename),path);
+   snprintf(OUTPUT,CF_BUFSIZE,"Suspicious filename %s in %s has no alphanumeric content (security)",CanonifyName(nodename),path);
    CfLog(cfsilent,OUTPUT,"");
    strcpy(newname,vbuff);
 
    for (sp = newname+strlen(path); *sp != '\0'; sp++)
       {
       if ((*sp > 126) || (*sp < 33))
-	 {
-	 *sp = 50 + (*sp / 50);  /* Create a visible ASCII interpretation */
-	 }
+  {
+  *sp = 50 + (*sp / 50);  /* Create a visible ASCII interpretation */
+  }
       }
 
    strcat(newname,".cf-nonalpha");
    
-   snprintf(OUTPUT,bufsize,"Renaming file %s to %s",vbuff,newname);
+   snprintf(OUTPUT,CF_BUFSIZE,"Renaming file %s to %s",vbuff,newname);
    CfLog(cfsilent,OUTPUT,"");
    
    if (rename(vbuff,newname) == -1)
@@ -141,7 +145,7 @@ if (strstr(nodename,".") && (EXTENSIONLIST != NULL))
    {
    if (cflstat(vbuff,&statbuf,ip) == -1)
       {
-      snprintf(OUTPUT,bufsize,"Couldn't examine %s - foreign filesystem?\n",vbuff);
+      snprintf(OUTPUT,CF_BUFSIZE,"Couldn't examine %s - foreign filesystem?\n",vbuff);
       CfLog(cfverbose,OUTPUT,"lstat");
       return true;
       }
@@ -149,26 +153,26 @@ if (strstr(nodename,".") && (EXTENSIONLIST != NULL))
    if (S_ISDIR(statbuf.st_mode))
       {
       if (strcmp(nodename,"...") == 0)
-	 {
-	 Verbose("Hidden directory ... found in %s\n",path);
-	 return true;
-	 }
-
+         {
+         Verbose("Hidden directory ... found in %s\n",path);
+         return true;
+         }
+      
       for (sp = nodename+strlen(nodename)-1; *sp != '.'; sp--)
-	 {
-	 }
-
+         {
+         }
+      
       if ((char *)sp != nodename) /* Don't get .dir */
-	 {
-	 sp++; /* Find file extension, look for known plain files  */
-
-	 if ((strlen(sp) > 0) && IsItemIn(EXTENSIONLIST,sp))
-	    {
-	    snprintf(OUTPUT,bufsize,"Suspicious directory %s in %s looks like plain file with extension .%s",nodename,path,sp);
-	    CfLog(cfsilent,OUTPUT,"");
-	    return false;
-	    }
-	 }
+         {
+         sp++; /* Find file extension, look for known plain files  */
+         
+         if ((strlen(sp) > 0) && IsItemIn(EXTENSIONLIST,sp))
+            {
+            snprintf(OUTPUT,CF_BUFSIZE,"Suspicious directory %s in %s looks like plain file with extension .%s",nodename,path,sp);
+            CfLog(cfsilent,OUTPUT,"");
+            return false;
+            }
+         }
       }
    }
 
@@ -181,32 +185,65 @@ for (sp = nodename; *sp != '\0'; sp++) /* Check for files like ".. ." */
       }
    }
 
-snprintf(OUTPUT,bufsize,"Suspicous looking file object \"%s\" masquerading as hidden file in %s\n",nodename,path);
-CfLog(cfsilent,OUTPUT,"");
-Debug("Filename looks suspicious\n"); 
 
 /* removed if (EXTENSIONLIST==NULL) mb */ 
 
 if (cflstat(vbuff,&statbuf,ip) == -1)
    {
-   snprintf(OUTPUT,bufsize,"Couldn't stat %s",vbuff);
+   snprintf(OUTPUT,CF_BUFSIZE,"Couldn't stat %s",vbuff);
    CfLog(cfverbose,OUTPUT,"lstat");
    return true;
    }
 
+if (statbuf.st_size == 0 && ! (VERBOSE||INFORM)) /* No sense in warning about empty files */
+   {
+   return false;
+   }
+ 
+snprintf(OUTPUT,CF_BUFSIZE,"Suspicious looking file object \"%s\" masquerading as hidden file in %s\n",nodename,path);
+CfLog(cfsilent,OUTPUT,"");
+Debug("Filename looks suspicious\n"); 
+ 
 if (S_ISLNK(statbuf.st_mode))
    {
-   snprintf(OUTPUT,bufsize,"   %s is a symbolic link\n",nodename);
+   snprintf(OUTPUT,CF_BUFSIZE,"   %s is a symbolic link\n",nodename);
    CfLog(cfsilent,OUTPUT,"");
    }
 else if (S_ISDIR(statbuf.st_mode))
    {
-   snprintf(OUTPUT,bufsize,"   %s is a directory\n",nodename);
+   snprintf(OUTPUT,CF_BUFSIZE,"   %s is a directory\n",nodename);
    CfLog(cfsilent,OUTPUT,"");
    }
 
-snprintf(OUTPUT,bufsize,"[%s] has size %ld and full mode %o\n",nodename,(unsigned long)(statbuf.st_size),(unsigned int)(statbuf.st_mode));
+snprintf(OUTPUT,CF_BUFSIZE,"[%s] has size %ld and full mode %o\n",nodename,(unsigned long)(statbuf.st_size),(unsigned int)(statbuf.st_mode));
 CfLog(cfsilent,OUTPUT,"");
  
 return true;
 }
+
+
+/********************************************************************/
+
+void RegisterRecursionRootDevice(dev_t device)
+
+{
+Debug("Registering root device as %d\n",device);
+ROOTDEVICE = device;
+}
+
+
+/********************************************************************/
+
+int DeviceChanged(dev_t thisdevice)
+
+{
+if (thisdevice == ROOTDEVICE)
+   {
+   return false;
+   }
+else
+   {
+   Verbose("Device change from %d to %d\n",ROOTDEVICE,thisdevice);
+   return true;
+   }
+}
